@@ -2,23 +2,43 @@ defmodule EEx.Engine do
   @moduledoc ~S"""
   Basic EEx engine that ships with Elixir.
 
-  An engine needs to implement three functions:
+  An engine needs to implement six functions:
 
-    * `init(opts)` - returns the initial buffer
+    * `init(opts)` - called at the beginning of every text
+      and it must return the initial state.
 
-    * `handle_body(quoted)` - receives the final built quoted
-      expression, should do final post-processing and return a
-      quoted expression.
+    * `handle_body(state)` - receives the state of the document
+      and it must return a quoted expression.
 
-    * `handle_text(buffer, text)` - it receives the buffer,
+    * `handle_text(state, text)` - it receives the state,
       the text and must return a new quoted expression.
 
-    * `handle_expr(buffer, marker, expr)` - it receives the buffer,
-      the marker, the expr and must return a new quoted expression.
+    * `handle_expr(state, marker, expr)` - it receives the state,
+      the marker, the expr and must return a new state.
+
+    * `handle_begin(state)` - called every time there a new state
+      is needed with an empty buffer. Typically called for do/end
+      blocks, case expressions, anonymous functions, etc
+
+    * `handle_end(state)` - opposite of `handle_begin(state)` and
+      it must return quoted expression
 
       The marker is what follows exactly after `<%`. For example,
       `<% foo %>` has an empty marker, but `<%= foo %>` has `"="`
-      as marker. The allowed markers so far are: `""` and `"="`.
+      as marker. The allowed markers so far are: 
+
+        * `""`
+        * `"="`
+        * `"/"`
+        * `"|"`
+
+      Markers `"/"` and `"|"` are only for use in custom EEx engines
+      and are not implemented by default. Using them without the
+      implementation raises `EEx.SyntaxError`.
+
+      If your engine does not implement all markers, please ensure that
+      `handle_expr/3` falls back to `EEx.Engine.handle_expr/3` 
+      to raise the proper error message.
 
       Read `handle_expr/3` below for more information about the markers
       implemented by default by this engine.
@@ -27,10 +47,14 @@ defmodule EEx.Engine do
   default implementations for the functions above.
   """
 
-  @callback init(Keyword.t) :: Macro.t
-  @callback handle_body(Macro.t) :: Macro.t
-  @callback handle_text(Macro.t, String.t) :: Macro.t
-  @callback handle_expr(Macro.t, String.t, Macro.t) :: Macro.t
+  @type state :: term
+
+  @callback init(opts :: keyword) :: state
+  @callback handle_body(state) :: Macro.t()
+  @callback handle_text(state, text :: String.t()) :: state
+  @callback handle_expr(state, marker :: String.t(), expr :: Macro.t()) :: state
+  @callback handle_begin(state) :: state
+  @callback handle_end(state) :: Macro.t()
 
   @doc false
   defmacro __using__(_) do
@@ -41,8 +65,16 @@ defmodule EEx.Engine do
         EEx.Engine.init(opts)
       end
 
-      def handle_body(body) do
-        EEx.Engine.handle_body(body)
+      def handle_body(quoted) do
+        EEx.Engine.handle_body(quoted)
+      end
+
+      def handle_begin(quoted) do
+        EEx.Engine.handle_begin(quoted)
+      end
+
+      def handle_end(quoted) do
+        EEx.Engine.handle_end(quoted)
       end
 
       def handle_text(buffer, text) do
@@ -53,7 +85,7 @@ defmodule EEx.Engine do
         EEx.Engine.handle_expr(buffer, marker, expr)
       end
 
-      defoverridable [handle_body: 1, handle_expr: 3, handle_text: 2, init: 1]
+      defoverridable EEx.Engine
     end
   end
 
@@ -72,25 +104,33 @@ defmodule EEx.Engine do
       end
 
   """
+  @spec handle_assign(Macro.t()) :: Macro.t()
   def handle_assign({:@, meta, [{name, _, atom}]}) when is_atom(name) and is_atom(atom) do
     line = meta[:line] || 0
-    quote line: line, do: EEx.Engine.fetch_assign!(var!(assigns), unquote(name))
+    quote(line: line, do: EEx.Engine.fetch_assign!(var!(assigns), unquote(name)))
   end
+
   def handle_assign(arg) do
     arg
   end
 
   @doc false
   # TODO: Raise on 2.0
+  @spec fetch_assign!(Access.t(), Access.key()) :: term | nil
   def fetch_assign!(assigns, key) do
     case Access.fetch(assigns, key) do
       {:ok, val} ->
         val
+
       :error ->
         keys = Enum.map(assigns, &elem(&1, 0))
-        IO.warn "assign @#{key} not available in EEx template. " <>
-                "Please ensure all assigns are given as options. " <>
-                "Available assigns: #{inspect keys}"
+
+        IO.warn(
+          "assign @#{key} not available in EEx template. " <>
+            "Please ensure all assigns are given as options. " <>
+            "Available assigns: #{inspect(keys)}"
+        )
+
         nil
     end
   end
@@ -102,6 +142,20 @@ defmodule EEx.Engine do
     ""
   end
 
+  @doc """
+  Returns an empty string as the new buffer.
+  """
+  def handle_begin(_previous) do
+    ""
+  end
+
+  @doc """
+  End of the new buffer.
+  """
+  def handle_end(quoted) do
+    quoted
+  end
+
   @doc """
   The default implementation simply returns the given expression.
   """
@@ -113,7 +167,7 @@ defmodule EEx.Engine do
   The default implementation simply concatenates text to the buffer.
   """
   def handle_text(buffer, text) do
-    quote do: unquote(buffer) <> unquote(text)
+    quote(do: unquote(buffer) <> unquote(text))
   end
 
   @doc """
@@ -121,10 +175,11 @@ defmodule EEx.Engine do
 
       <% Elixir expression - inline with output %>
       <%= Elixir expression - replace with result %>
+      <%/ Elixir expression - raise EEx.SyntaxError, to be implemented by custom engines %>
+      <%| Elixir expression - raise EEx.SyntaxError, to be implemented by custom engines %>
 
   All other markers are not implemented by this engine.
   """
-  @spec handle_expr(Macro.t, String.t, Macro.t) :: Macro.t
   def handle_expr(buffer, "=", expr) do
     quote do
       tmp1 = unquote(buffer)
@@ -139,4 +194,9 @@ defmodule EEx.Engine do
       tmp2
     end
   end
+
+  def handle_expr(_buffer, marker, _expr) when marker in ["/", "|"] do
+    raise EEx.SyntaxError,
+          "unsupported EEx syntax <%#{marker} %> (the syntax is valid but not supported by the current EEx engine)"
+  end
 end
