@@ -6,16 +6,16 @@ defmodule Bitwise do
   operators. For example:
 
       iex> use Bitwise
-      iex> bnot 1   # named
+      iex> bnot(1) # named
       -2
-      iex> 1 &&& 1  # operator
+      iex> 1 &&& 1 # operator
       1
 
   If you prefer to use only operators or skip them, you can
   pass the following options:
 
-    * `:only_operators` - include only operators
-    * `:skip_operators` - skip operators
+    * `:only_operators` - includes only operators
+    * `:skip_operators` - skips operators
 
   For example:
 
@@ -29,7 +29,10 @@ defmodule Bitwise do
   All bitwise macros can be used in guards:
 
       iex> use Bitwise
-      iex> odd? = fn int when band(int, 1) == 1 -> true; _ -> false end
+      iex> odd? = fn
+      ...>   int when band(int, 1) == 1 -> true
+      ...>   _ -> false
+      ...> end
       iex> odd?.(1)
       true
 
@@ -37,14 +40,17 @@ defmodule Bitwise do
 
   @doc false
   defmacro __using__(options) do
-    except = cond do
-      Keyword.get(options, :only_operators) ->
-        [bnot: 1, band: 2, bor: 2, bxor: 2, bsl: 2, bsr: 2]
-      Keyword.get(options, :skip_operators) ->
-        [~~~: 1, &&&: 2, |||: 2, ^^^: 2, <<<: 2, >>>: 2]
-      true ->
-        []
-    end
+    except =
+      cond do
+        Keyword.get(options, :only_operators) ->
+          [bnot: 1, band: 2, bor: 2, bxor: 2, bsl: 2, bsr: 2]
+
+        Keyword.get(options, :skip_operators) ->
+          [~~~: 1, &&&: 2, |||: 2, ^^^: 2, <<<: 2, >>>: 2]
+
+        true ->
+          []
+      end
 
     quote do
       import Bitwise, except: unquote(except)
@@ -61,7 +67,7 @@ defmodule Bitwise do
 
   """
   defmacro bnot(expr) do
-    quote do: :erlang.bnot(unquote(expr))
+    quote(do: :erlang.bnot(unquote(expr)))
   end
 
   @doc """
@@ -74,7 +80,7 @@ defmodule Bitwise do
 
   """
   defmacro ~~~expr do
-    quote do: :erlang.bnot(unquote(expr))
+    quote(do: :erlang.bnot(unquote(expr)))
   end
 
   @doc """
@@ -85,7 +91,7 @@ defmodule Bitwise do
 
   """
   defmacro band(left, right) do
-    quote do: :erlang.band(unquote(left), unquote(right))
+    quote(do: :erlang.band(unquote(left), unquote(right)))
   end
 
   @doc """
@@ -96,7 +102,7 @@ defmodule Bitwise do
 
   """
   defmacro left &&& right do
-    quote do: :erlang.band(unquote(left), unquote(right))
+    quote(do: :erlang.band(unquote(left), unquote(right)))
   end
 
   @doc """
@@ -107,7 +113,7 @@ defmodule Bitwise do
 
   """
   defmacro bor(left, right) do
-    quote do: :erlang.bor(unquote(left), unquote(right))
+    quote(do: :erlang.bor(unquote(left), unquote(right)))
   end
 
   @doc """
@@ -118,7 +124,7 @@ defmodule Bitwise do
 
   """
   defmacro left ||| right do
-    quote do: :erlang.bor(unquote(left), unquote(right))
+    quote(do: :erlang.bor(unquote(left), unquote(right)))
   end
 
   @doc """
@@ -129,7 +135,7 @@ defmodule Bitwise do
 
   """
   defmacro bxor(left, right) do
-    quote do: :erlang.bxor(unquote(left), unquote(right))
+    quote(do: :erlang.bxor(unquote(left), unquote(right)))
   end
 
   @doc """
@@ -140,7 +146,7 @@ defmodule Bitwise do
 
   """
   defmacro left ^^^ right do
-    quote do: :erlang.bxor(unquote(left), unquote(right))
+    quote(do: :erlang.bxor(unquote(left), unquote(right)))
   end
 
   @doc """
@@ -157,7 +163,7 @@ defmodule Bitwise do
 
   """
   defmacro bsl(left, right) do
-    quote do: :erlang.bsl(unquote(left), unquote(right))
+    quote(do: :erlang.bsl(unquote(left), unquote(right)))
   end
 
   @doc """
@@ -174,7 +180,7 @@ defmodule Bitwise do
 
   """
   defmacro left <<< right do
-    quote do: :erlang.bsl(unquote(left), unquote(right))
+    quote(do: :erlang.bsl(unquote(left), unquote(right)))
   end
 
   @doc """
@@ -191,7 +197,7 @@ defmodule Bitwise do
 
   """
   defmacro bsr(left, right) do
-    quote do: :erlang.bsr(unquote(left), unquote(right))
+    quote(do: :erlang.bsr(unquote(left), unquote(right)))
   end
 
   @doc """
@@ -208,6 +214,6 @@ defmodule Bitwise do
 
   """
   defmacro left >>> right do
-    quote do: :erlang.bsr(unquote(left), unquote(right))
+    quote(do: :erlang.bsr(unquote(left), unquote(right)))
   end
 end
