@@ -1,18 +1,24 @@
 defprotocol List.Chars do
   @moduledoc ~S"""
-  The List.Chars protocol is responsible for
-  converting a structure to a list (only if applicable).
-  The only function required to be implemented is
-  `to_charlist` which does the conversion.
+  The `List.Chars` protocol is responsible for
+  converting a structure to a charlist (only if applicable).
 
-  The `to_charlist` function automatically imported
-  by Kernel invokes this protocol.
+  The only function required to be implemented is
+  `to_charlist/1` which does the conversion.
+
+  The `to_charlist/1` function automatically imported
+  by `Kernel` invokes this protocol.
   """
 
+  @doc """
+  Converts `term` to a charlist.
+  """
+  @spec to_charlist(t) :: charlist
   def to_charlist(term)
 
-  # TODO: Deprecate by v1.5
   @doc false
+  # TODO: Remove by 2.0
+  @deprecated "Use List.Chars.to_charlist/1 instead"
   Kernel.def to_char_list(term) do
     __MODULE__.to_charlist(term)
   end
@@ -32,9 +38,9 @@ defimpl List.Chars, for: BitString do
 
   def to_charlist(term) do
     raise Protocol.UndefinedError,
-             protocol: @protocol,
-                value: term,
-          description: "cannot convert a bitstring to a charlist"
+      protocol: @protocol,
+      value: term,
+      description: "cannot convert a bitstring to a charlist"
   end
 end
 
