@@ -10,10 +10,10 @@ import(Meta, Ref, Opts, E) ->
     case keyfind(only, Opts) of
       {only, functions} ->
         {Added1, Funs} = import_functions(Meta, Ref, Opts, E),
-        {Funs, keydelete(Ref, ?m(E, macros)), Added1};
+        {Funs, keydelete(Ref, ?key(E, macros)), Added1};
       {only, macros} ->
         {Added2, Macs} = import_macros(true, Meta, Ref, Opts, E),
-        {keydelete(Ref, ?m(E, functions)), Macs, Added2};
+        {keydelete(Ref, ?key(E, functions)), Macs, Added2};
       {only, List} when is_list(List) ->
         {Added1, Funs} = import_functions(Meta, Ref, Opts, E),
         {Added2, Macs} = import_macros(false, Meta, Ref, Opts, E),
@@ -28,15 +28,19 @@ import(Meta, Ref, Opts, E) ->
   {Functions, Macros}.
 
 import_functions(Meta, Ref, Opts, E) ->
-  calculate(Meta, Ref, Opts, ?m(E, functions), ?m(E, file), fun() ->
+  calculate(Meta, Ref, Opts, ?key(E, functions), ?key(E, file), fun() ->
     get_functions(Ref)
   end).
 
 import_macros(Force, Meta, Ref, Opts, E) ->
-  calculate(Meta, Ref, Opts, ?m(E, macros), ?m(E, file), fun() ->
-    case Force of
-      true  -> get_macros(Meta, Ref, E);
-      false -> get_optional_macros(Ref)
+  calculate(Meta, Ref, Opts, ?key(E, macros), ?key(E, file), fun() ->
+    case fetch_macros(Ref) of
+      {ok, Macros} ->
+        Macros;
+      error when Force ->
+        elixir_errors:form_error(Meta, ?key(E, file), ?MODULE, {no_macros, Ref});
+      error ->
+        []
     end
   end).
 
@@ -54,7 +58,7 @@ record_warn(Meta, Ref, Opts, Added, E) ->
       _ -> []
     end,
 
-  elixir_lexical:record_import(Ref, Only, ?line(Meta), Added and Warn, ?m(E, lexical_tracker)).
+  elixir_lexical:record_import(Ref, Only, ?line(Meta), Added and Warn, ?key(E, lexical_tracker)).
 
 %% Calculates the imports based on only and except
 
@@ -63,24 +67,27 @@ calculate(Meta, Key, Opts, Old, File, Existing) ->
     {only, Only} when is_list(Only) ->
       ok = ensure_keyword_list(Meta, File, Only, only),
       case keyfind(except, Opts) of
-        false -> ok;
+        false ->
+          ok;
         _ ->
-          elixir_errors:compile_error(Meta, File,
-            ":only and :except can only be given together to import"
-            " when :only is either :functions or :macros")
+          elixir_errors:form_error(Meta, File, ?MODULE, only_and_except_given)
       end,
       case Only -- get_exports(Key) of
         [{Name, Arity} | _] ->
-          Tuple = {invalid_import, {Key, Name, Arity}},
-          elixir_errors:form_error(Meta, File, ?MODULE, Tuple);
+          elixir_errors:form_error(Meta, File, ?MODULE, {invalid_import, {Key, Name, Arity}});
         _ ->
           intersection(Only, Existing())
       end;
     _ ->
       case keyfind(except, Opts) of
-        false -> remove_underscored(Existing());
+        false ->
+          remove_underscored(Existing());
         {except, Except} when is_list(Except) ->
           ok = ensure_keyword_list(Meta, File, Except, except),
+          %% We are not checking existence of exports listed in :except option
+          %% on purpose: to support backwards compatible code.
+          %% For example, "import String, except: [trim: 1]"
+          %% should work across all Elixir versions.
           case keyfind(Key, Old) of
             false -> remove_underscored(Existing()) -- Except;
             {Key, OldImports} -> OldImports -- Except
@@ -103,11 +110,7 @@ calculate(Meta, Key, Opts, Old, File, Existing) ->
 %% Retrieve functions and macros from modules
 
 get_exports(Module) ->
-  try
-    Module:'__info__'(functions) ++ Module:'__info__'(macros)
-  catch
-    error:undef -> Module:module_info(exports)
-  end.
+  get_functions(Module) ++ get_macros(Module).
 
 get_functions(Module) ->
   try
@@ -116,24 +119,19 @@ get_functions(Module) ->
     error:undef -> Module:module_info(exports)
   end.
 
-get_macros(Meta, Module, E) ->
-  try
-    Module:'__info__'(macros)
-  catch
-    error:undef ->
-      Tuple = {no_macros, Module},
-      elixir_errors:form_error(Meta, ?m(E, file), ?MODULE, Tuple)
+get_macros(Module) ->
+  case fetch_macros(Module) of
+    {ok, Macros} ->
+      Macros;
+    error ->
+      []
   end.
 
-get_optional_macros(Module)  ->
-  case code:ensure_loaded(Module) of
-    {module, Module} ->
-      try
-        Module:'__info__'(macros)
-      catch
-        error:undef -> []
-      end;
-    {error, _} -> []
+fetch_macros(Module) ->
+  try
+    {ok, Module:'__info__'(macros)}
+  catch
+    error:undef -> error
   end.
 
 %% VALIDATION HELPERS
@@ -141,8 +139,7 @@ get_optional_macros(Module)  ->
 ensure_no_special_form_conflict(Meta, File, Key, [{Name, Arity} | T]) ->
   case special_form(Name, Arity) of
     true  ->
-      Tuple = {special_form_conflict, {Key, Name, Arity}},
-      elixir_errors:form_error(Meta, File, ?MODULE, Tuple);
+      elixir_errors:form_error(Meta, File, ?MODULE, {special_form_conflict, {Key, Name, Arity}});
     false ->
       ensure_no_special_form_conflict(Meta, File, Key, T)
   end;
@@ -151,18 +148,26 @@ ensure_no_special_form_conflict(_Meta, _File, _Key, []) -> ok.
 
 ensure_keyword_list(_Meta, _File, [], _Kind) -> ok;
 
-ensure_keyword_list(Meta, File, [{Key, _} | Rest], Kind) when is_atom(Key) ->
+ensure_keyword_list(Meta, File, [{Key, Value} | Rest], Kind) when is_atom(Key), is_integer(Value) ->
   ensure_keyword_list(Meta, File, Rest, Kind);
 
 ensure_keyword_list(Meta, File, _Other, Kind) ->
-  elixir_errors:compile_error(Meta, File, "invalid :~s option for import, expected a keyword list", [Kind]).
+  elixir_errors:form_error(Meta, File, ?MODULE, {invalid_option, Kind}).
 
 %% ERROR HANDLING
 
+format_error(only_and_except_given) ->
+  ":only and :except can only be given together to import "
+  "when :only is either :functions or :macros";
+
 format_error({invalid_import, {Receiver, Name, Arity}}) ->
-  io_lib:format("cannot import ~ts.~ts/~B because it doesn't exist",
+  io_lib:format("cannot import ~ts.~ts/~B because it is undefined or private",
     [elixir_aliases:inspect(Receiver), Name, Arity]);
 
+format_error({invalid_option, Option}) ->
+  Message = "invalid :~s option for import, expected a keyword list with integer values",
+  io_lib:format(Message, [Option]);
+
 format_error({special_form_conflict, {Receiver, Name, Arity}}) ->
   io_lib:format("cannot import ~ts.~ts/~B because it conflicts with Elixir special forms",
     [elixir_aliases:inspect(Receiver), Name, Arity]);
@@ -220,6 +225,7 @@ special_form('import', 1) -> true;
 special_form('import', 2) -> true;
 special_form('__ENV__', 0) -> true;
 special_form('__CALLER__', 0) -> true;
+special_form('__STACKTRACE__', 0) -> true;
 special_form('__MODULE__', 0) -> true;
 special_form('__DIR__', 0) -> true;
 special_form('__aliases__', _) -> true;
@@ -233,6 +239,6 @@ special_form('for', _) -> true;
 special_form('with', _) -> true;
 special_form('cond', 1) -> true;
 special_form('case', 2) -> true;
-special_form('try', 2) -> true;
+special_form('try', 1) -> true;
 special_form('receive', 1) -> true;
 special_form(_, _) -> false.
