@@ -1,19 +1,19 @@
 %% Module responsible for tracking lexical information.
 -module(elixir_lexical).
--export([run/3, dest/1,
+-export([run/2, set_file/2, reset_file/1,
   record_alias/4, record_alias/2,
   record_import/6, record_import/5,
   record_remote/3, record_remote/6,
-  format_error/1
+  record_struct/3, format_error/1
 ]).
 -include("elixir.hrl").
 
 -define(tracker, 'Elixir.Kernel.LexicalTracker').
 
-run(File, Dest, Callback) ->
-  case elixir_compiler:get_opt(internal) of
+run(File, Callback) ->
+  case elixir_config:get(bootstrap) of
     false ->
-      {ok, Pid} = ?tracker:start_link(Dest),
+      {ok, Pid} = ?tracker:start_link(),
       try Callback(Pid) of
         Res ->
           warn_unused_aliases(File, Pid),
@@ -27,9 +27,6 @@ run(File, Dest, Callback) ->
       Callback(nil)
   end.
 
-dest(nil) -> nil;
-dest(Pid) -> ?tracker:dest(Pid).
-
 %% RECORD
 
 record_alias(Module, Line, Warn, Ref) ->
@@ -50,6 +47,17 @@ record_remote(Module, EnvFunction, Ref) ->
 record_remote(Module, Function, Arity, EnvFunction, Line, Ref) ->
   if_tracker(Ref, fun(Pid) -> ?tracker:remote_dispatch(Pid, Module, {Function, Arity}, Line, mode(EnvFunction)), ok end).
 
+record_struct(Module, Line, Ref) ->
+  if_tracker(Ref, fun(Pid) -> ?tracker:remote_struct(Pid, Module, Line), ok end).
+
+%% EXTERNAL SOURCES
+
+set_file(File, Ref) ->
+  if_tracker(Ref, fun(Pid) -> ?tracker:set_file(Pid, File), ok end).
+
+reset_file(Ref) ->
+  if_tracker(Ref, fun(Pid) -> ?tracker:reset_file(Pid), ok end).
+
 %% HELPERS
 
 mode(nil) -> compile;
