@@ -1,5 +1,5 @@
 defmodule Undef do
-  def undef do
+  def undef() do
     %__MODULE__{}
   end
 end
