@@ -1,4 +1,4 @@
-Code.require_file "test_helper.exs", __DIR__
+Code.require_file("test_helper.exs", __DIR__)
 
 defmodule PathTest do
   use ExUnit.Case, async: true
@@ -7,16 +7,16 @@ defmodule PathTest do
 
   import PathHelpers
 
-  if :file.native_name_encoding == :utf8 do
-    test "wildcard with utf8" do
+  if :file.native_name_encoding() == :utf8 do
+    test "wildcard with UTF-8" do
       File.mkdir_p(tmp_path("héllò"))
       assert Path.wildcard(tmp_path("héllò")) == [tmp_path("héllò")]
     after
-      File.rm_rf tmp_path("héllò")
+      File.rm_rf(tmp_path("héllò"))
     end
   end
 
-  test "wildcard" do
+  test "wildcard/2" do
     hello = tmp_path("wildcard/.hello")
     world = tmp_path("wildcard/.hello/world")
     File.mkdir_p(world)
@@ -29,19 +29,27 @@ defmodule PathTest do
     assert Path.wildcard(tmp_path("wildcard/**/*"), match_dot: true) == [hello, world]
     assert Path.wildcard(tmp_path("wildcard/?hello/world"), match_dot: true) == [world]
   after
-    File.rm_rf tmp_path("wildcard")
+    File.rm_rf(tmp_path("wildcard"))
   end
 
-  if windows? do
-    test "relative win" do
-      assert Path.relative("C:/usr/local/bin")    == "usr/local/bin"
+  test "wildcard/2 raises on null byte" do
+    assert_raise ArgumentError, ~r/null byte/, fn -> Path.wildcard("foo\0bar") end
+  end
+
+  describe "Windows" do
+    @describetag :windows
+
+    test "relative/1" do
+      assert Path.relative("C:/usr/local/bin") == "usr/local/bin"
       assert Path.relative("C:\\usr\\local\\bin") == "usr\\local\\bin"
-      assert Path.relative("C:usr\\local\\bin")   == "usr\\local\\bin"
+      assert Path.relative("C:usr\\local\\bin") == "usr\\local\\bin"
 
-      assert Path.relative("/usr/local/bin")   == "usr/local/bin"
-      assert Path.relative("usr/local/bin")    == "usr/local/bin"
+      assert Path.relative("/usr/local/bin") == "usr/local/bin"
+      assert Path.relative("usr/local/bin") == "usr/local/bin"
       assert Path.relative("../usr/local/bin") == "../usr/local/bin"
+    end
 
+    test "relative_to/2" do
       assert Path.relative_to("D:/usr/local/foo", "D:/usr/") == "local/foo"
       assert Path.relative_to("D:/usr/local/foo", "d:/usr/") == "local/foo"
       assert Path.relative_to("d:/usr/local/foo", "D:/usr/") == "local/foo"
@@ -51,58 +59,63 @@ defmodule PathTest do
       assert Path.relative_to("D:/usr/local/foo", "D:") == "D:/usr/local/foo"
     end
 
-    test "type win" do
-      assert Path.type("C:/usr/local/bin")    == :absolute
+    test "type/1" do
+      assert Path.type("C:/usr/local/bin") == :absolute
       assert Path.type('C:\\usr\\local\\bin') == :absolute
-      assert Path.type("C:usr\\local\\bin")   == :volumerelative
+      assert Path.type("C:usr\\local\\bin") == :volumerelative
 
-      assert Path.type("/usr/local/bin")   == :volumerelative
-      assert Path.type('usr/local/bin')    == :relative
+      assert Path.type("/usr/local/bin") == :volumerelative
+      assert Path.type('usr/local/bin') == :relative
       assert Path.type("../usr/local/bin") == :relative
     end
 
-    test "split win" do
+    test "split/1" do
       assert Path.split("C:\\foo\\bar") == ["c:/", "foo", "bar"]
       assert Path.split("C:/foo/bar") == ["c:/", "foo", "bar"]
     end
-  else
-    test "relative Unix" do
-      assert Path.relative("/usr/local/bin")   == "usr/local/bin"
-      assert Path.relative("usr/local/bin")    == "usr/local/bin"
+  end
+
+  describe "Unix" do
+    @describetag :unix
+
+    test "relative/1" do
+      assert Path.relative("/usr/local/bin") == "usr/local/bin"
+      assert Path.relative("usr/local/bin") == "usr/local/bin"
       assert Path.relative("../usr/local/bin") == "../usr/local/bin"
+      assert Path.relative("/") == "."
+      assert Path.relative('/') == "."
       assert Path.relative(['/usr', ?/, "local/bin"]) == "usr/local/bin"
     end
 
-    test "type Unix" do
-      assert Path.type("/usr/local/bin")   == :absolute
-      assert Path.type("usr/local/bin")    == :relative
+    test "type/1" do
+      assert Path.type("/usr/local/bin") == :absolute
+      assert Path.type("usr/local/bin") == :relative
       assert Path.type("../usr/local/bin") == :relative
 
-      assert Path.type('/usr/local/bin')   == :absolute
-      assert Path.type('usr/local/bin')    == :relative
+      assert Path.type('/usr/local/bin') == :absolute
+      assert Path.type('usr/local/bin') == :relative
       assert Path.type('../usr/local/bin') == :relative
 
-      assert Path.type(['/usr/', 'local/bin'])   == :absolute
-      assert Path.type(['usr/', 'local/bin'])    == :relative
+      assert Path.type(['/usr/', 'local/bin']) == :absolute
+      assert Path.type(['usr/', 'local/bin']) == :relative
       assert Path.type(['../usr', '/local/bin']) == :relative
     end
   end
 
-  test "relative to cwd" do
-    assert Path.relative_to_cwd(__ENV__.file) ==
-           Path.relative_to(__ENV__.file, System.cwd!)
+  test "relative_to_cwd/1" do
+    assert Path.relative_to_cwd(__ENV__.file) == Path.relative_to(__ENV__.file, System.cwd!())
 
     assert Path.relative_to_cwd(to_charlist(__ENV__.file)) ==
-           Path.relative_to(to_charlist(__ENV__.file), to_charlist(System.cwd!))
+             Path.relative_to(to_charlist(__ENV__.file), to_charlist(System.cwd!()))
   end
 
-  test "absname" do
-    assert (Path.absname("/") |> strip_drive_letter_if_windows) == "/"
-    assert (Path.absname("/foo") |> strip_drive_letter_if_windows) == "/foo"
-    assert (Path.absname("/./foo") |> strip_drive_letter_if_windows) == "/foo"
-    assert (Path.absname("/foo/bar") |> strip_drive_letter_if_windows) == "/foo/bar"
-    assert (Path.absname("/foo/bar/") |> strip_drive_letter_if_windows)  == "/foo/bar"
-    assert (Path.absname("/foo/bar/../bar")  |> strip_drive_letter_if_windows) == "/foo/bar/../bar"
+  test "absname/1,2" do
+    assert Path.absname("/") |> strip_drive_letter_if_windows == "/"
+    assert Path.absname("/foo") |> strip_drive_letter_if_windows == "/foo"
+    assert Path.absname("/./foo") |> strip_drive_letter_if_windows == "/foo"
+    assert Path.absname("/foo/bar") |> strip_drive_letter_if_windows == "/foo/bar"
+    assert Path.absname("/foo/bar/") |> strip_drive_letter_if_windows == "/foo/bar"
+    assert Path.absname("/foo/bar/../bar") |> strip_drive_letter_if_windows == "/foo/bar/../bar"
 
     assert Path.absname("bar", "/foo") == "/foo/bar"
     assert Path.absname("bar/", "/foo") == "/foo/bar"
@@ -112,47 +125,53 @@ defmodule PathTest do
     assert Path.absname(["bar/", ?., ?., ["/bar"]], "/foo") == "/foo/bar/../bar"
   end
 
-  test "expand path with user home" do
-    home = System.user_home! |> Path.absname
+  test "expand/1,2 with user home" do
+    home = System.user_home!() |> Path.absname()
 
     assert home == Path.expand("~")
     assert home == Path.expand('~')
-    assert is_binary Path.expand("~/foo")
-    assert is_binary Path.expand('~/foo')
+    assert is_binary(Path.expand("~/foo"))
+    assert is_binary(Path.expand('~/foo'))
 
     assert Path.expand("~/file") == Path.join(home, "file")
     assert Path.expand("~/file", "whatever") == Path.join(home, "file")
     assert Path.expand("file", Path.expand("~")) == Path.expand("~/file")
     assert Path.expand("file", "~") == Path.join(home, "file")
-    assert Path.expand("~file") == Path.join(System.cwd!, "file")
+    assert Path.expand("~file") == Path.join(System.cwd!(), "file")
   end
 
-  test "expand path" do
-    assert (Path.expand("/") |> strip_drive_letter_if_windows) == "/"
-    assert (Path.expand("/foo/../..") |> strip_drive_letter_if_windows) == "/"
-    assert (Path.expand("/foo") |> strip_drive_letter_if_windows) == "/foo"
-    assert (Path.expand("/./foo") |> strip_drive_letter_if_windows) == "/foo"
-    assert (Path.expand("/../foo") |> strip_drive_letter_if_windows) == "/foo"
-    assert (Path.expand("/foo/bar") |> strip_drive_letter_if_windows) == "/foo/bar"
-    assert (Path.expand("/foo/bar/") |> strip_drive_letter_if_windows) == "/foo/bar"
-    assert (Path.expand("/foo/bar/.") |> strip_drive_letter_if_windows)== "/foo/bar"
-    assert (Path.expand("/foo/bar/../bar") |> strip_drive_letter_if_windows) == "/foo/bar"
+  test "expand/1,2" do
+    assert Path.expand("/") |> strip_drive_letter_if_windows == "/"
+    assert Path.expand("/foo/../..") |> strip_drive_letter_if_windows == "/"
+    assert Path.expand("/foo") |> strip_drive_letter_if_windows == "/foo"
+    assert Path.expand("/./foo") |> strip_drive_letter_if_windows == "/foo"
+    assert Path.expand("/../foo") |> strip_drive_letter_if_windows == "/foo"
+    assert Path.expand("/foo/bar") |> strip_drive_letter_if_windows == "/foo/bar"
+    assert Path.expand("/foo/bar/") |> strip_drive_letter_if_windows == "/foo/bar"
+    assert Path.expand("/foo/bar/.") |> strip_drive_letter_if_windows == "/foo/bar"
+    assert Path.expand("/foo/bar/../bar") |> strip_drive_letter_if_windows == "/foo/bar"
 
-    assert (Path.expand("bar", "/foo") |> strip_drive_letter_if_windows)== "/foo/bar"
-    assert (Path.expand("bar/", "/foo") |> strip_drive_letter_if_windows)== "/foo/bar"
-    assert (Path.expand("bar/.", "/foo") |> strip_drive_letter_if_windows)== "/foo/bar"
-    assert (Path.expand("bar/../bar", "/foo") |> strip_drive_letter_if_windows)== "/foo/bar"
-    assert (Path.expand("../bar/../bar", "/foo/../foo/../foo") |> strip_drive_letter_if_windows) == "/bar"
+    assert Path.expand("bar", "/foo") |> strip_drive_letter_if_windows == "/foo/bar"
+    assert Path.expand("bar/", "/foo") |> strip_drive_letter_if_windows == "/foo/bar"
+    assert Path.expand("bar/.", "/foo") |> strip_drive_letter_if_windows == "/foo/bar"
+    assert Path.expand("bar/../bar", "/foo") |> strip_drive_letter_if_windows == "/foo/bar"
 
-    assert "/bar" ==
-      (Path.expand(['..', ?/, "bar/../bar"], '/foo/../foo/../foo') |> strip_drive_letter_if_windows)
+    drive_letter =
+      Path.expand("../bar/../bar", "/foo/../foo/../foo") |> strip_drive_letter_if_windows
 
-    assert (Path.expand("/..") |> strip_drive_letter_if_windows) == "/"
+    assert drive_letter == "/bar"
+
+    drive_letter =
+      Path.expand(['..', ?/, "bar/../bar"], '/foo/../foo/../foo') |> strip_drive_letter_if_windows
+
+    assert "/bar" == drive_letter
+
+    assert Path.expand("/..") |> strip_drive_letter_if_windows == "/"
 
     assert Path.expand("bar/../bar", "foo") == Path.expand("foo/bar")
   end
 
-  test "relative to" do
+  test "relative_to/2" do
     assert Path.relative_to("/usr/local/foo", "/usr/local") == "foo"
     assert Path.relative_to("/usr/local/foo", "/") == "usr/local/foo"
     assert Path.relative_to("/usr/local/foo", "/etc") == "/usr/local/foo"
@@ -166,14 +185,14 @@ defmodule PathTest do
     assert Path.relative_to(["usr", ?/, 'local/foo'], 'usr/local') == "foo"
   end
 
-  test "rootname" do
+  test "rootname/2" do
     assert Path.rootname("~/foo/bar.ex", ".ex") == "~/foo/bar"
     assert Path.rootname("~/foo/bar.exs", ".ex") == "~/foo/bar.exs"
     assert Path.rootname("~/foo/bar.old.ex", ".ex") == "~/foo/bar.old"
     assert Path.rootname([?~, '/foo/bar', ".old.ex"], '.ex') == "~/foo/bar.old"
   end
 
-  test "extname" do
+  test "extname/1" do
     assert Path.extname("foo.erl") == ".erl"
     assert Path.extname("~/foo/bar") == ""
 
@@ -181,7 +200,7 @@ defmodule PathTest do
     assert Path.extname('~/foo/bar') == ""
   end
 
-  test "dirname" do
+  test "dirname/1" do
     assert Path.dirname("/foo/bar.ex") == "/foo"
     assert Path.dirname("foo/bar.ex") == "foo"
 
@@ -191,7 +210,7 @@ defmodule PathTest do
     assert Path.dirname([?~, "/foo", '/bar.ex']) == "~/foo"
   end
 
-  test "basename" do
+  test "basename/1,2" do
     assert Path.basename("foo") == "foo"
     assert Path.basename("/foo/bar") == "bar"
     assert Path.basename("/") == ""
@@ -203,7 +222,7 @@ defmodule PathTest do
     assert Path.basename([?~, "/for/bar", '.old.ex'], ".ex") == "bar.old"
   end
 
-  test "join" do
+  test "join/1" do
     assert Path.join([""]) == ""
     assert Path.join(["foo"]) == "foo"
     assert Path.join(["/", "foo", "bar"]) == "/foo/bar"
@@ -211,14 +230,16 @@ defmodule PathTest do
     assert Path.join(['/foo/', "/bar/"]) == "/foo/bar"
     assert Path.join(["/", ""]) == "/"
     assert Path.join(["/", "", "bar"]) == "/bar"
+    assert Path.join(['foo', [?b, "a", ?r]]) == "foo/bar"
+    assert Path.join([[?f, 'o', "o"]]) == "foo"
   end
 
-  test "join two" do
+  test "join/2" do
     assert Path.join("/foo", "bar") == "/foo/bar"
     assert Path.join("~", "foo") == "~/foo"
 
-    assert Path.join("", "bar")  == "bar"
-    assert Path.join("bar", "")  == "bar"
+    assert Path.join("", "bar") == "bar"
+    assert Path.join("bar", "") == "bar"
     assert Path.join("", "/bar") == "bar"
     assert Path.join("/bar", "") == "/bar"
 
@@ -228,16 +249,17 @@ defmodule PathTest do
     assert Path.join("/foo", "./bar") == "/foo/./bar"
 
     assert Path.join([?/, "foo"], "./bar") == "/foo/./bar"
+    assert Path.join(["/foo", "bar"], ["fiz", "buz"]) == "/foobar/fizbuz"
   end
 
-  test "split" do
+  test "split/1" do
     assert Path.split("") == []
     assert Path.split("foo") == ["foo"]
     assert Path.split("/foo/bar") == ["/", "foo", "bar"]
     assert Path.split([?/, "foo/bar"]) == ["/", "foo", "bar"]
   end
 
-  if windows? do
+  if windows?() do
     defp strip_drive_letter_if_windows([_d, ?: | rest]), do: rest
     defp strip_drive_letter_if_windows(<<_d, ?:, rest::binary>>), do: rest
   else
