@@ -5,10 +5,11 @@ defmodule ExUnit.AssertionError do
 
   @no_value :ex_unit_no_meaningful_value
 
-  defexception left:    @no_value,
-               right:   @no_value,
+  defexception left: @no_value,
+               right: @no_value,
                message: @no_value,
-               expr:    @no_value
+               expr: @no_value,
+               args: @no_value
 
   @doc """
   Indicates no meaningful value for a field.
@@ -17,12 +18,10 @@ defmodule ExUnit.AssertionError do
     @no_value
   end
 
+  @impl true
   def message(exception) do
-    "\n\n" <>
-      ExUnit.Formatter.format_assertion_error(exception, :infinity, &formatter/2, "")
+    "\n\n" <> ExUnit.Formatter.format_assertion_error(exception)
   end
-
-  defp formatter(_, msg), do: msg
 end
 
 defmodule ExUnit.MultiError do
@@ -30,11 +29,12 @@ defmodule ExUnit.MultiError do
   Raised to signal multiple errors happened in a test case.
   """
 
-  defexception [errors: []]
+  defexception errors: []
 
-  def message(exception) do
+  @impl true
+  def message(%{errors: errors}) do
     "got the following errors:\n\n" <>
-      Enum.map_join(exception, "\n\n", fn {kind, error, stack} ->
+      Enum.map_join(errors, "\n\n", fn {kind, error, stack} ->
         Exception.format_banner(kind, error, stack)
       end)
   end
@@ -47,48 +47,48 @@ defmodule ExUnit.Assertions do
 
   In general, a developer will want to use the general
   `assert` macro in tests. This macro introspects your code
-  and provide good reporting whenever there is a failure.
+  and provides good reporting whenever there is a failure.
   For example, `assert some_fun() == 10` will fail (assuming
-  `some_fun()` returns 13):
+  `some_fun()` returns `13`):
 
       Comparison (using ==) failed in:
-      code: some_fun() == 10
-      lhs:  13
-      rhs:  10
+      code:  some_fun() == 10
+      left:  13
+      right: 10
 
   This module also provides other convenience functions
   like `assert_in_delta` and `assert_raise` to easily handle
-  other common cases such as checking a floating point number
+  other common cases such as checking a floating-point number
   or handling exceptions.
   """
 
   @doc """
   Asserts its argument is a truthy value.
 
-  `assert` instrospects the underlying expression and provide
-  good  reporting whenever there is a failure. For example,
+  `assert` introspects the underlying expression and provides
+  good reporting whenever there is a failure. For example,
   if the expression uses the comparison operator, the message
   will show the values of the two sides. The assertion
 
-      assert 1+2+3+4 > 15
+      assert 1 + 2 + 3 + 4 > 15
 
    will fail with the message:
 
       Assertion with > failed
-      code: 1+2+3+4 > 15
-      lhs:  10
-      rhs:  15
+      code:  1 + 2 + 3 + 4 > 15
+      left:  10
+      right: 15
 
   Similarly, if a match expression is given, it will report
   any failure in terms of that match. Given
 
-      assert [one] = [two]
+      assert [1] = [2]
 
   you'll see:
 
       match (=) failed
-      code: [one] = [two]
-      rhs:  [2]
+      code:  [1] = [2]
+      right: [2]
 
   Keep in mind that `assert` does not change its semantics
   based on the expression. In other words, the expression
@@ -102,78 +102,88 @@ defmodule ExUnit.Assertions do
   `Kernel.match?/2`.
   """
   defmacro assert({:=, _, [left, right]} = assertion) do
-    code = Macro.escape(assertion)
+    code = escape_quoted(:assert, assertion)
 
-    left = Macro.expand(left, __CALLER__)
+    left = expand_pattern(left, __CALLER__)
     vars = collect_vars_from_pattern(left)
-    pins = collect_pins_from_pattern(left)
+    pins = collect_pins_from_pattern(left, Macro.Env.vars(__CALLER__))
 
     # If the match works, we need to check if the value
     # is not nil nor false. We need to rewrite the if
     # to avoid silly warnings though.
     return =
-      no_warning(quote do
-        case right do
-          x when x in [nil, false] ->
-            raise ExUnit.AssertionError,
-              expr: expr,
-              message: "Expected truthy, got #{inspect right}"
-          _ ->
-            :ok
+      suppress_warning(
+        quote do
+          case right do
+            x when x in [nil, false] ->
+              raise ExUnit.AssertionError,
+                expr: expr,
+                message: "Expected truthy, got #{inspect(right)}"
+
+            _ ->
+              :ok
+          end
         end
-      end)
+      )
+
+    match_expr =
+      suppress_warning(
+        quote do
+          case right do
+            unquote(left) ->
+              unquote(return)
+              unquote(vars)
+
+            _ ->
+              raise ExUnit.AssertionError,
+                right: right,
+                expr: expr,
+                message: "match (=) failed" <> ExUnit.Assertions.__pins__(unquote(pins))
+          end
+        end
+      )
 
     quote do
       right = unquote(right)
-      expr  = unquote(code)
-      unquote(vars) =
-        case right do
-          unquote(left) ->
-            unquote(return)
-            unquote(vars)
-          _ ->
-            raise ExUnit.AssertionError,
-              right: right,
-              expr: expr,
-              message: "match (=) failed" <>
-                       ExUnit.Assertions.__pins__(unquote(pins))
-        end
+      expr = unquote(code)
+      unquote(vars) = unquote(match_expr)
       right
     end
   end
 
   defmacro assert({:match?, meta, [left, right]} = assertion) do
-    code   = Macro.escape(assertion)
+    code = escape_quoted(:assert, assertion)
     match? = {:match?, meta, [left, Macro.var(:right, __MODULE__)]}
-    pins   = collect_pins_from_pattern(left)
+    pins = collect_pins_from_pattern(left, Macro.Env.vars(__CALLER__))
 
     quote do
       right = unquote(right)
+
       assert unquote(match?),
         right: right,
         expr: unquote(code),
-        message: "match (match?) failed" <>
-                 ExUnit.Assertions.__pins__(unquote(pins))
+        message: "match (match?) failed" <> ExUnit.Assertions.__pins__(unquote(pins))
     end
   end
 
   defmacro assert(assertion) do
-    case translate_assertion(assertion) do
-      nil ->
-        quote do
-          value = unquote(assertion)
+    if translated = translate_assertion(:assert, assertion, __CALLER__) do
+      translated
+    else
+      {args, value} = extract_args(assertion, __CALLER__)
 
-          unless value do
-            raise ExUnit.AssertionError,
-              expr: unquote(Macro.escape(assertion)),
-              message: "Expected truthy, got #{inspect value}"
-          end
+      quote do
+        value = unquote(value)
 
-          value
+        unless value do
+          raise ExUnit.AssertionError,
+            args: unquote(args),
+            expr: unquote(escape_quoted(:assert, assertion)),
+            message: "Expected truthy, got #{inspect(value)}"
         end
 
-      value ->
         value
+      end
     end
   end
 
@@ -199,37 +209,40 @@ defmodule ExUnit.Assertions do
 
   """
   defmacro refute({:match?, meta, [left, right]} = assertion) do
-    code   = Macro.escape(assertion)
+    code = escape_quoted(:refute, assertion)
     match? = {:match?, meta, [left, Macro.var(:right, __MODULE__)]}
-    pins   = collect_pins_from_pattern(left)
+    pins = collect_pins_from_pattern(left, Macro.Env.vars(__CALLER__))
 
     quote do
       right = unquote(right)
+
       refute unquote(match?),
         right: right,
         expr: unquote(code),
-        message: "match (match?) succeeded, but should have failed" <>
-                 ExUnit.Assertions.__pins__(unquote(pins))
+        message:
+          "match (match?) succeeded, but should have failed" <>
+            ExUnit.Assertions.__pins__(unquote(pins))
     end
   end
 
   defmacro refute(assertion) do
-    case translate_assertion({:!, [], [assertion]}) do
-      nil ->
-        quote do
-          value = unquote(assertion)
+    if translated = translate_assertion(:refute, assertion, __CALLER__) do
+      {:!, [], [translated]}
+    else
+      {args, value} = extract_args(assertion, __CALLER__)
 
-          if value do
-            raise ExUnit.AssertionError,
-              expr: unquote(Macro.escape(assertion)),
-              message: "Expected false or nil, got #{inspect value}"
-          end
+      quote do
+        value = unquote(value)
 
-          value
+        if value do
+          raise ExUnit.AssertionError,
+            args: unquote(args),
+            expr: unquote(escape_quoted(:refute, assertion)),
+            message: "Expected false or nil, got #{inspect(value)}"
         end
 
-      value ->
-        {:!, [], [value]}
+        value
+      end
     end
   end
 
@@ -237,38 +250,105 @@ defmodule ExUnit.Assertions do
 
   @operator [:==, :<, :>, :<=, :>=, :===, :=~, :!==, :!=, :in]
 
-  defp translate_assertion({operator, meta, [left, right]} = expr) when operator in @operator  do
-    expr = Macro.escape(expr)
-    call = {operator, meta, [Macro.var(:left, __MODULE__), Macro.var(:right, __MODULE__)]}
-    quote do
-      left  = unquote(left)
-      right = unquote(right)
-      assert unquote(call),
-             left: left,
-             right: right,
-             expr: unquote(expr),
-             message: unquote("Assertion with #{operator} failed")
-    end
+  defp translate_assertion(:assert, {operator, meta, [_, _]} = expr, caller)
+       when operator in @operator do
+    left = Macro.var(:left, __MODULE__)
+    right = Macro.var(:right, __MODULE__)
+    call = {operator, meta, [left, right]}
+    equality_check? = operator in [:<, :>, :!==, :!=]
+    message = "Assertion with #{operator} failed"
+    translate_operator(:assert, expr, call, message, equality_check?, caller)
   end
 
-  defp translate_assertion({:!, _, [{operator, meta, [left, right]} = expr]}) when operator in @operator do
-    expr = Macro.escape(expr)
-    call = {operator, meta, [Macro.var(:left, __MODULE__), Macro.var(:right, __MODULE__)]}
-    quote do
-      left  = unquote(left)
-      right = unquote(right)
-      assert not(unquote(call)),
-             left: left,
-             right: right,
-             expr: unquote(expr),
-             message: unquote("Refute with #{operator} failed")
-    end
+  defp translate_assertion(:refute, {operator, meta, [_, _]} = expr, caller)
+       when operator in @operator do
+    left = Macro.var(:left, __MODULE__)
+    right = Macro.var(:right, __MODULE__)
+    call = {:not, meta, [{operator, meta, [left, right]}]}
+    equality_check? = operator in [:<=, :>=, :===, :==, :=~]
+    message = "Refute with #{operator} failed"
+    translate_operator(:refute, expr, call, message, equality_check?, caller)
   end
 
-  defp translate_assertion(_expected) do
+  defp translate_assertion(_kind, _expected, _caller) do
     nil
   end
 
+  defp translate_operator(kind, {_, _, [left, right]} = expr, call, message, true, _caller) do
+    expr = escape_quoted(kind, expr)
+
+    quote do
+      left = unquote(left)
+      right = unquote(right)
+
+      if ExUnit.Assertions.__equal__?(left, right) do
+        assert false,
+          left: left,
+          expr: unquote(expr),
+          message: unquote(message <> ", both sides are exactly equal")
+      else
+        assert unquote(call),
+          left: left,
+          right: right,
+          expr: unquote(expr),
+          message: unquote(message)
+      end
+    end
+  end
+
+  defp translate_operator(kind, {_, _, [left, right]} = expr, call, message, false, _caller) do
+    expr = escape_quoted(kind, expr)
+
+    quote do
+      left = unquote(left)
+      right = unquote(right)
+
+      assert unquote(call),
+        left: left,
+        right: right,
+        expr: unquote(expr),
+        message: unquote(message)
+    end
+  end
+
+  @doc false
+  def __equal__?(left, right) do
+    left === right
+  end
+
+  defp escape_quoted(kind, expr) do
+    Macro.escape({kind, [], [expr]}, prune_metadata: true)
+  end
+
+  defp extract_args({root, meta, [_ | _] = args} = expr, env) do
+    arity = length(args)
+
+    reserved? =
+      is_atom(root) and (Macro.special_form?(root, arity) or Macro.operator?(root, arity))
+
+    all_quoted_literals? = Enum.all?(args, &Macro.quoted_literal?/1)
+
+    case Macro.expand_once(expr, env) do
+      ^expr when not reserved? and not all_quoted_literals? ->
+        vars = for i <- 1..arity, do: Macro.var(:"arg#{i}", __MODULE__)
+
+        quoted =
+          quote do
+            {unquote_splicing(vars)} = {unquote_splicing(args)}
+            unquote({root, meta, vars})
+          end
+
+        {vars, quoted}
+
+      other ->
+        {ExUnit.AssertionError.no_value(), other}
+    end
+  end
+
+  defp extract_args(expr, _env) do
+    {ExUnit.AssertionError.no_value(), expr}
+  end
+
   ## END HELPERS
 
   @doc """
@@ -289,9 +369,10 @@ defmodule ExUnit.Assertions do
   end
 
   @doc """
-  Asserts that a message matching `pattern` was or is going to be received.
+  Asserts that a message matching `pattern` was or is going to be received
+  within the `timeout` period, specified in milliseconds.
 
-  Unlike `assert_received`, it has a default timeout
+  Unlike `assert_received`, it has a default `timeout`
   of 100 milliseconds.
 
   The `pattern` argument must be a match pattern. Flunks with `failure_message`
@@ -313,10 +394,12 @@ defmodule ExUnit.Assertions do
       assert_receive {:count, ^x}
 
   """
-  defmacro assert_receive(pattern,
-                          timeout \\ Application.fetch_env!(:ex_unit, :assert_receive_timeout),
-                          failure_message \\ nil) do
-    do_assert_receive(pattern, timeout, failure_message, __CALLER__)
+  defmacro assert_receive(
+             pattern,
+             timeout \\ Application.fetch_env!(:ex_unit, :assert_receive_timeout),
+             failure_message \\ nil
+           ) do
+    assert_receive(pattern, timeout, failure_message, __CALLER__)
   end
 
   @doc """
@@ -330,10 +413,10 @@ defmodule ExUnit.Assertions do
 
   ## Examples
 
-      send self, :hello
+      send self(), :hello
       assert_received :hello
 
-      send self, :bye
+      send self(), :bye
       assert_received :hello, "Oh No!"
       ** (ExUnit.AssertionError) Oh No!
       Process mailbox:
@@ -341,42 +424,77 @@ defmodule ExUnit.Assertions do
 
   You can also match against specific patterns:
 
-      send self, {:hello, "world"}
+      send self(), {:hello, "world"}
       assert_received {:hello, _}
 
   """
   defmacro assert_received(pattern, failure_message \\ nil) do
-    do_assert_receive(pattern, 0, failure_message, __CALLER__)
+    assert_receive(pattern, 0, failure_message, __CALLER__)
   end
 
-  defp do_assert_receive(pattern, timeout, failure_message, caller) do
+  defp assert_receive(pattern, timeout, failure_message, caller) do
     binary = Macro.to_string(pattern)
 
     # Expand before extracting metadata
-    pattern = Macro.expand(pattern, caller)
-    vars = collect_vars_from_pattern(pattern)
-    pins = collect_pins_from_pattern(pattern)
+    caller = Macro.Env.to_match(caller)
+    expanded = expand_pattern(pattern, caller)
+    vars = collect_vars_from_pattern(expanded)
+    pins = collect_pins_from_pattern(expanded, Macro.Env.vars(caller))
 
     pattern =
       case pattern do
         {:when, meta, [left, right]} ->
           {:when, meta, [quote(do: unquote(left) = received), right]}
+
         left ->
           quote(do: unquote(left) = received)
       end
 
+    quoted_pattern =
+      quote do
+        case message do
+          unquote(pattern) ->
+            _ = unquote(vars)
+            true
+
+          _ ->
+            false
+        end
+      end
+
+    pattern_finder =
+      quote do
+        fn message ->
+          unquote(suppress_warning(quoted_pattern))
+        end
+      end
+
+    timeout =
+      if is_integer(timeout) do
+        timeout
+      else
+        quote do: ExUnit.Assertions.__timeout__(unquote(timeout))
+      end
+
+    failure_message =
+      failure_message ||
+        quote do
+          ExUnit.Assertions.__timeout__(
+            unquote(binary),
+            unquote(pins),
+            unquote(pattern_finder),
+            timeout
+          )
+        end
+
     quote do
       timeout = unquote(timeout)
 
       {received, unquote(vars)} =
         receive do
-          unquote(pattern) ->
-            {received, unquote(vars)}
+          unquote(pattern) -> {received, unquote(vars)}
         after
-          timeout ->
-            failure_message = unquote(failure_message) || "No message matching #{unquote(binary)} after #{timeout}ms."
-            flunk(failure_message <> ExUnit.Assertions.__pins__(unquote(pins))
-                          <> ExUnit.Assertions.__mailbox__(self()))
+          timeout -> flunk(unquote(failure_message))
         end
 
       received
@@ -387,74 +505,127 @@ defmodule ExUnit.Assertions do
   @max_mailbox_length 10
 
   @doc false
-  def __mailbox__(pid) do
-    {:messages, messages} = Process.info(pid, :messages)
-    length = length(messages)
-    mailbox =
-      messages
-      |> Enum.take(@max_mailbox_length)
-      |> Enum.map_join(@indent, &inspect/1)
-    mailbox_message(length, @indent <> mailbox)
+  def __timeout__(timeout) when is_integer(timeout) and timeout >= 0, do: timeout
+
+  def __timeout__(timeout),
+    do: raise(ArgumentError, "timeout must be a non-negative integer, got: #{inspect(timeout)}")
+
+  @doc false
+  def __timeout__(binary, pins, pattern_finder, timeout) do
+    {:messages, messages} = Process.info(self(), :messages)
+
+    if Enum.any?(messages, pattern_finder) do
+      """
+      Found message matching #{binary} after #{timeout}ms.
+
+      This means the message was delivered too close to the timeout value, you may want to either:
+
+        1. Give an increased timeout to `assert_receive/2`
+        2. Increase the default timeout to all `assert_receive` in your
+           test_helper.exs by setting ExUnit.configure(assert_receive_timeout: ...)
+      """
+    else
+      "No message matching #{binary} after #{timeout}ms." <>
+        __pins__(pins) <> format_mailbox(messages)
+    end
   end
 
   @doc false
   def __pins__([]), do: ""
+
   def __pins__(pins) do
     content =
       pins
       |> Enum.reverse()
       |> Enum.map_join(@indent, fn {name, var} -> "#{name} = #{inspect(var)}" end)
+
     "\nThe following variables were pinned:" <> @indent <> content
   end
 
-  defp mailbox_message(0, _mailbox), do: "\nThe process mailbox is empty."
-  defp mailbox_message(length, mailbox) when length > 10 do
-    "\nProcess mailbox:" <> mailbox
-      <> "\nShowing only #{@max_mailbox_length} of #{length} messages."
+  defp format_mailbox(messages) do
+    length = length(messages)
+
+    mailbox =
+      messages
+      |> Enum.take(@max_mailbox_length)
+      |> Enum.map_join(@indent, &inspect/1)
+
+    mailbox_message(length, @indent <> mailbox)
   end
+
+  defp mailbox_message(0, _mailbox), do: "\nThe process mailbox is empty."
+
+  defp mailbox_message(length, mailbox) when length > 10 do
+    "\nProcess mailbox:" <>
+      mailbox <> "\nShowing only #{@max_mailbox_length} of #{length} messages."
+  end
+
   defp mailbox_message(_length, mailbox) do
     "\nProcess mailbox:" <> mailbox
   end
 
-  defp collect_pins_from_pattern(expr) do
+  defp collect_pins_from_pattern(expr, vars) do
     {_, pins} =
       Macro.prewalk(expr, [], fn
-        {:^, _, [{name, _, _} = var]}, acc ->
-          {:ok, [{name, var} | acc]}
+        {:^, _, [{name, _, nil} = var]}, acc ->
+          if {name, nil} in vars do
+            {:ok, [{name, var} | acc]}
+          else
+            {:ok, acc}
+          end
+
         form, acc ->
           {form, acc}
       end)
+
     Enum.uniq_by(pins, &elem(&1, 0))
   end
 
   defp collect_vars_from_pattern({:when, _, [left, right]}) do
     pattern = collect_vars_from_pattern(left)
+
     for {name, _, context} = var <- collect_vars_from_pattern(right),
-      Enum.any?(pattern, &match?({^name, _, ^context}, &1)),
-      into: pattern,
-      do: var
+        Enum.any?(pattern, &match?({^name, _, ^context}, &1)),
+        into: pattern,
+        do: var
   end
 
   defp collect_vars_from_pattern(expr) do
     Macro.prewalk(expr, [], fn
       {:::, _, [left, _]}, acc ->
         {[left], acc}
+
       {skip, _, [_]}, acc when skip in [:^, :@] ->
         {:ok, acc}
+
       {:_, _, context}, acc when is_atom(context) ->
         {:ok, acc}
+
       {name, meta, context}, acc when is_atom(name) and is_atom(context) ->
         {:ok, [{name, [generated: true] ++ meta, context} | acc]}
+
       node, acc ->
         {node, acc}
     end)
     |> elem(1)
   end
 
-  defp no_warning({name, meta, [expr, [do: clauses]]}) do
-    clauses = Enum.map clauses, fn {:->, meta, args} ->
-      {:->, [generated: true] ++ Keyword.put(meta, :line, -1), args}
-    end
+  defp expand_pattern(expr, caller) do
+    Macro.prewalk(expr, fn
+      {var, _, context} = node when is_atom(var) and is_atom(context) ->
+        node
+
+      other ->
+        Macro.expand(other, caller)
+    end)
+  end
+
+  defp suppress_warning({name, meta, [expr, [do: clauses]]}) do
+    clauses =
+      Enum.map(clauses, fn {:->, meta, args} ->
+        {:->, [generated: true] ++ meta, args}
+      end)
+
     {name, meta, [expr, [do: clauses]]}
   end
 
@@ -470,23 +641,25 @@ defmodule ExUnit.Assertions do
       end
 
       assert_raise RuntimeError, ~r/^today's lucky number is 0\.\d+!$/, fn ->
-        raise "today's lucky number is #{:rand.uniform}!"
+        raise "today's lucky number is #{:rand.uniform()}!"
       end
+
   """
   def assert_raise(exception, message, function) when is_function(function) do
     error = assert_raise(exception, function)
 
-    is_match = cond do
-      is_binary(message) -> Exception.message(error) == message
-      Regex.regex?(message) -> Exception.message(error) =~ message
-    end
+    match? =
+      cond do
+        is_binary(message) -> Exception.message(error) == message
+        Regex.regex?(message) -> Exception.message(error) =~ message
+      end
 
-    msg = "Wrong message for #{inspect exception}\n" <>
-          "Expected:\n" <>
-          "  #{inspect message}\n" <>
-          "Got:\n" <>
-          "  #{inspect Exception.message(error)}"
-    assert is_match, message: msg
+    message =
+      "Wrong message for #{inspect(exception)}\n" <>
+        "expected:\n  #{inspect(message)}\n" <>
+        "actual:\n" <> "  #{inspect(Exception.message(error))}"
+
+    unless match?, do: flunk(message)
 
     error
   end
@@ -507,42 +680,72 @@ defmodule ExUnit.Assertions do
       function.()
     rescue
       error ->
-        stacktrace = System.stacktrace
         name = error.__struct__
 
         cond do
           name == exception ->
+            check_error_message(name, error)
             error
+
           name == ExUnit.AssertionError ->
-            reraise(error, stacktrace)
+            reraise(error, __STACKTRACE__)
+
           true ->
-            reraise ExUnit.AssertionError, [message: "Expected exception #{inspect exception} but got #{inspect name} (#{Exception.message(error)})"], stacktrace
+            message =
+              "Expected exception #{inspect(exception)} " <>
+                "but got #{inspect(name)} (#{Exception.message(error)})"
+
+            reraise ExUnit.AssertionError, [message: message], __STACKTRACE__
         end
     else
-      _ -> flunk "Expected exception #{inspect exception} but nothing was raised"
+      _ -> flunk("Expected exception #{inspect(exception)} but nothing was raised")
     end
   end
 
+  defp check_error_message(module, error) do
+    module.message(error)
+  catch
+    kind, reason ->
+      message =
+        "Got exception #{inspect(module)} but it failed to produce a message with:\n\n" <>
+          Exception.format(kind, reason, __STACKTRACE__)
+
+      flunk(message)
+  end
+
   @doc """
   Asserts that `value1` and `value2` differ by no more than `delta`.
 
+  This difference is inclusive, so the test will pass if the difference
+  and the `delta` are equal.
 
   ## Examples
 
       assert_in_delta 1.1, 1.5, 0.2
-      assert_in_delta 10, 15, 4
+      assert_in_delta 10, 15, 2
+      assert_in_delta 10, 15, 5
 
   """
-  def assert_in_delta(value1, value2, delta, message \\ nil) do
+  def assert_in_delta(value1, value2, delta, message \\ nil)
+
+  def assert_in_delta(_, _, delta, _) when delta < 0 do
+    raise ArgumentError, "delta must always be a positive number, got: #{inspect(delta)}"
+  end
+
+  def assert_in_delta(value1, value2, delta, message) do
     diff = abs(value1 - value2)
-    message = message ||
-      "Expected the difference between #{inspect value1} and " <>
-      "#{inspect value2} (#{inspect diff}) to be less than #{inspect delta}"
-    assert diff < delta, message
+
+    message =
+      message ||
+        "Expected the difference between #{inspect(value1)} and " <>
+          "#{inspect(value2)} (#{inspect(diff)}) to be less than or equal to #{inspect(delta)}"
+
+    assert diff <= delta, message
   end
 
   @doc """
   Asserts `expression` will throw a value.
+
   Returns the thrown value or fails otherwise.
 
   ## Examples
@@ -556,11 +759,19 @@ defmodule ExUnit.Assertions do
 
   @doc """
   Asserts `expression` will exit.
-  Returns the exit status/message or fails otherwise.
+
+  Returns the exit status/message of the current process or fails otherwise.
 
   ## Examples
 
       assert catch_exit(exit 1) == 1
+      
+  To assert exits from linked processes started from the test, trap exits
+  with `Process.flag/2` and assert the exit message with `assert_received/2`.
+
+      Process.flag(:trap_exit, true)
+      pid = spawn_link(fn -> Process.exit(self(), :normal) end)
+      assert_receive {:EXIT, ^pid, :normal}
 
   """
   defmacro catch_exit(expression) do
@@ -569,6 +780,7 @@ defmodule ExUnit.Assertions do
 
   @doc """
   Asserts `expression` will cause an error.
+
   Returns the error or fails otherwise.
 
   ## Examples
@@ -584,10 +796,10 @@ defmodule ExUnit.Assertions do
     quote do
       try do
         _ = unquote(expr)
-        flunk "Expected to catch #{unquote(kind)}, got nothing"
+        flunk("Expected to catch #{unquote(kind)}, got nothing")
       rescue
         e in [ExUnit.AssertionError] ->
-          reraise(e, System.stacktrace)
+          reraise(e, __STACKTRACE__)
       catch
         unquote(kind), we_got -> we_got
       end
@@ -608,7 +820,7 @@ defmodule ExUnit.Assertions do
 
   @doc """
   Asserts that a message matching `pattern` was not received (and won't be received)
-  within the `timeout` period.
+  within the `timeout` period, specified in milliseconds.
 
   The `pattern` argument must be a match pattern. Flunks with `failure_message`
   if a message matching `pattern` is received.
@@ -622,9 +834,11 @@ defmodule ExUnit.Assertions do
       refute_receive :bye, 1000
 
   """
-  defmacro refute_receive(pattern,
-                          timeout \\ Application.fetch_env!(:ex_unit, :refute_receive_timeout),
-                          failure_message \\ nil) do
+  defmacro refute_receive(
+             pattern,
+             timeout \\ Application.fetch_env!(:ex_unit, :refute_receive_timeout),
+             failure_message \\ nil
+           ) do
     do_refute_receive(pattern, timeout, failure_message)
   end
 
@@ -639,10 +853,10 @@ defmodule ExUnit.Assertions do
 
   ## Examples
 
-      send self, :hello
+      send self(), :hello
       refute_received :bye
 
-      send self, :hello
+      send self(), :hello
       refute_received :hello, "Oh No!"
       ** (ExUnit.AssertionError) Oh No!
       Process mailbox:
@@ -665,23 +879,29 @@ defmodule ExUnit.Assertions do
     end
   end
 
- defp refute_receive_clause(pattern, nil) do
-  binary = Macro.to_string(pattern)
-  quote do
-    unquote(pattern) = actual ->
-      flunk "Unexpectedly received message #{inspect actual} (which matched #{unquote binary})"
+  defp refute_receive_clause(pattern, nil) do
+    binary = Macro.to_string(pattern)
+
+    quote do
+      unquote(pattern) = actual ->
+        flunk(
+          "Unexpectedly received message #{inspect(actual)} (which matched #{unquote(binary)})"
+        )
     end
   end
 
   defp refute_receive_clause(pattern, failure_message) do
     quote do
-      unquote(pattern) -> flunk unquote(failure_message)
+      unquote(pattern) -> flunk(unquote(failure_message))
     end
   end
 
   @doc """
   Asserts `value1` and `value2` are not within `delta`.
 
+  This difference is exclusive, so the test will fail if the difference
+  and the delta are equal.
+
   If you supply `message`, information about the values will
   automatically be appended to it.
 
@@ -693,13 +913,17 @@ defmodule ExUnit.Assertions do
   """
   def refute_in_delta(value1, value2, delta, message \\ nil) do
     diff = abs(value1 - value2)
-    message = if message do
-      message <> " (difference between #{inspect value1} " <>
-      "and #{inspect value2} is less than #{inspect delta})"
-    else
-      "Expected the difference between #{inspect value1} and " <>
-      "#{inspect value2} (#{inspect diff}) to be more than #{inspect delta}"
-    end
+
+    message =
+      if message do
+        message <>
+          " (difference between #{inspect(value1)} " <>
+          "and #{inspect(value2)} is less than #{inspect(delta)})"
+      else
+        "Expected the difference between #{inspect(value1)} and " <>
+          "#{inspect(value2)} (#{inspect(diff)}) to be more than #{inspect(delta)}"
+      end
+
     refute diff < delta, message
   end
 
@@ -712,7 +936,7 @@ defmodule ExUnit.Assertions do
 
   """
   @spec flunk :: no_return
-  @spec flunk(String.t) :: no_return
+  @spec flunk(String.t()) :: no_return
   def flunk(message \\ "Flunked!") when is_binary(message) do
     assert false, message: message
   end
