@@ -1,4 +1,4 @@
-Code.require_file "../test_helper.exs", __DIR__
+Code.require_file("../test_helper.exs", __DIR__)
 
 defmodule ExUnit.AssertionsTest.Value do
   def tuple, do: {2, 1}
@@ -6,11 +6,23 @@ defmodule ExUnit.AssertionsTest.Value do
   def truthy, do: true
 end
 
-alias ExUnit.AssertionsTest.Value
+defmodule ExUnit.AssertionsTest.BrokenError do
+  defexception [:message]
+
+  @impl true
+  def message(_) do
+    raise "error"
+  end
+end
+
+alias ExUnit.AssertionsTest.{BrokenError, Value}
 
 defmodule ExUnit.AssertionsTest do
   use ExUnit.Case, async: true
 
+  defmacro sigil_l({:<<>>, _, [string]}, _), do: Code.string_to_quoted!(string, [])
+  defmacro argless_macro(), do: raise("should not be invoked")
+
   defmacrop assert_ok(arg) do
     quote do
       assert {:ok, val} = ok(unquote(arg))
@@ -18,11 +30,11 @@ defmodule ExUnit.AssertionsTest do
   end
 
   test "assert inside macro" do
-    assert_ok 42
+    assert_ok(42)
   end
 
   test "assert with true value" do
-    true = assert Value.truthy
+    true = assert Value.truthy()
   end
 
   test "assert with message when value is false" do
@@ -36,14 +48,36 @@ defmodule ExUnit.AssertionsTest do
 
   test "assert when value evaluates to false" do
     try do
-      "This should never be tested" = assert Value.falsy
+      "This should never be tested" = assert Value.falsy()
     rescue
       error in [ExUnit.AssertionError] ->
-        "Value.falsy()" = error.expr |> Macro.to_string
+        "assert(Value.falsy())" = error.expr |> Macro.to_string()
         "Expected truthy, got false" = error.message
     end
   end
 
+  test "assert arguments in special form" do
+    true =
+      assert (case :ok do
+                :ok -> true
+              end)
+  end
+
+  test "assert arguments semantics on function call" do
+    x = 1
+    true = assert not_equal(x = 2, x)
+    2 = x
+  end
+
+  test "assert arguments are not kept for operators" do
+    try do
+      "This should never be tested" = assert not Value.truthy()
+    rescue
+      error in [ExUnit.AssertionError] ->
+        false = is_list(error.args)
+    end
+  end
+
   test "assert with equality" do
     try do
       "This should never be tested" = assert 1 + 1 == 1
@@ -51,7 +85,7 @@ defmodule ExUnit.AssertionsTest do
       error in [ExUnit.AssertionError] ->
         1 = error.right
         2 = error.left
-        "1 + 1 == 1" = error.expr |> Macro.to_string
+        "assert(1 + 1 == 1)" = error.expr |> Macro.to_string()
     end
   end
 
@@ -62,166 +96,248 @@ defmodule ExUnit.AssertionsTest do
       error in [ExUnit.AssertionError] ->
         1 = error.left
         2 = error.right
-        "1 == 1 + 1" = error.expr |> Macro.to_string
+        "assert(1 == 1 + 1)" = error.expr |> Macro.to_string()
     end
   end
 
+  test "assert exposes nested macro variables in matches" do
+    assert ~l(a) = 1
+    assert a == 1
+
+    assert {~l(b), ~l(c)} = {2, 3}
+    assert b == 2
+    assert c == 3
+  end
+
+  test "assert does not expand variables" do
+    assert argless_macro = 1
+    assert argless_macro == 1
+  end
+
   test "refute when value is false" do
     false = refute false
   end
 
   test "refute when value evaluates to true" do
     try do
-      refute Value.truthy
+      refute Value.truthy()
       raise "refute was supposed to fail"
     rescue
       error in [ExUnit.AssertionError] ->
-        "Value.truthy()"   = error.expr |> Macro.to_string
+        "refute(Value.truthy())" = Macro.to_string(error.expr)
         "Expected false or nil, got true" = error.message
     end
   end
 
   test "assert match when equal" do
-    {2, 1} = (assert {2, 1} = Value.tuple)
+    {2, 1} = assert {2, 1} = Value.tuple()
   end
 
   test "assert match with pinned variable" do
     a = 1
-    {2, 1} = (assert {2, ^a} = Value.tuple)
+    {2, 1} = assert {2, ^a} = Value.tuple()
 
     try do
-      assert {^a, 1} = Value.tuple
+      assert {^a, 1} = Value.tuple()
     rescue
       error in [ExUnit.AssertionError] ->
-        "match (=) failed\n" <>
-        "The following variables were pinned:\n" <>
-        "  a = 1" = error.message
-        "{^a, 1} = Value.tuple()" = Macro.to_string(error.expr)
+        "match (=) failed\n" <> "The following variables were pinned:\n" <> "  a = 1" =
+          error.message
+
+        "assert({^a, 1} = Value.tuple())" = Macro.to_string(error.expr)
+    end
+  end
+
+  test "assert match with pinned variable from another context" do
+    var!(a, Elixir) = 1
+    {2, 1} = assert {2, ^var!(a, Elixir)} = Value.tuple()
+
+    try do
+      assert {^var!(a, Elixir), 1} = Value.tuple()
+    rescue
+      error in [ExUnit.AssertionError] ->
+        "match (=) failed" = error.message
+        "assert({^var!(a, Elixir), 1} = Value.tuple())" = Macro.to_string(error.expr)
     end
   end
 
   test "assert match?" do
-    true = assert match?({2, 1}, Value.tuple)
+    true = assert match?({2, 1}, Value.tuple())
 
     try do
       "This should never be tested" = assert match?({:ok, _}, error(true))
     rescue
       error in [ExUnit.AssertionError] ->
         "match (match?) failed" = error.message
-        "match?({:ok, _}, error(true))" = Macro.to_string(error.expr)
+        "assert(match?({:ok, _}, error(true)))" = Macro.to_string(error.expr)
         "{:error, true}" = Macro.to_string(error.right)
     end
   end
 
   test "refute match?" do
-    false = refute match?({1, 1}, Value.tuple)
+    false = refute match?({1, 1}, Value.tuple())
 
     try do
       "This should never be tested" = refute match?({:error, _}, error(true))
     rescue
       error in [ExUnit.AssertionError] ->
         "match (match?) succeeded, but should have failed" = error.message
-        "match?({:error, _}, error(true))" = Macro.to_string(error.expr)
+        "refute(match?({:error, _}, error(true)))" = Macro.to_string(error.expr)
         "{:error, true}" = Macro.to_string(error.right)
     end
   end
 
   test "assert match? with pinned variable" do
     a = 1
+
     try do
-      "This should never be tested" = assert(match?({^a, 1}, Value.tuple))
+      "This should never be tested" = assert(match?({^a, 1}, Value.tuple()))
     rescue
       error in [ExUnit.AssertionError] ->
-        "match (match?) failed\n" <>
-        "The following variables were pinned:\n" <>
-        "  a = 1" = error.message
-        "match?({^a, 1}, Value.tuple())" = Macro.to_string(error.expr)
+        "match (match?) failed\nThe following variables were pinned:\n  a = 1" = error.message
+
+        "assert(match?({^a, 1}, Value.tuple()))" = Macro.to_string(error.expr)
     end
   end
 
   test "refute match? with pinned variable" do
     a = 2
+
     try do
-      "This should never be tested" = refute(match?({^a, 1}, Value.tuple))
+      "This should never be tested" = refute(match?({^a, 1}, Value.tuple()))
     rescue
       error in [ExUnit.AssertionError] ->
-        "match (match?) succeeded, but should have failed\n" <>
-        "The following variables were pinned:\n" <>
-        "  a = 2" = error.message
-        "match?({^a, 1}, Value.tuple())" = Macro.to_string(error.expr)
+        """
+        match (match?) succeeded, but should have failed
+        The following variables were pinned:
+          a = 2\
+        """ = error.message
+
+        "refute(match?({^a, 1}, Value.tuple()))" = Macro.to_string(error.expr)
     end
   end
 
   test "assert receive waits" do
-    parent = self
-    spawn fn -> send parent, :hello end
+    parent = self()
+    spawn(fn -> send(parent, :hello) end)
     :hello = assert_receive :hello
   end
 
+  test "assert receive accepts custom failure message" do
+    send(self(), :hello)
+    assert_receive message, 0, "failure message"
+    :hello = message
+  end
+
+  test "assert receive with message in mailbox after timeout, but before reading mailbox tells user to increase timeout" do
+    parent = self()
+    # This is testing a race condition, so it's not
+    # guaranteed this works under all loads of the system
+    timeout = 100
+    spawn(fn -> Process.send_after(parent, :hello, timeout) end)
+
+    try do
+      assert_receive :hello, timeout
+    rescue
+      error in [ExUnit.AssertionError] ->
+        true =
+          error.message =~ "Found message matching :hello after 100ms" or
+            error.message =~ "No message matching :hello after 100ms"
+    end
+  end
+
+  test "assert_receive exposes nested macro variables" do
+    send(self(), {:hello})
+    assert_receive {~l(a)}, 0, "failure message"
+
+    assert a == :hello
+  end
+
+  test "assert_receive raises on invalid timeout" do
+    timeout = ok(1)
+
+    try do
+      assert_receive {~l(a)}, timeout
+    rescue
+      error in [ArgumentError] ->
+        "timeout must be a non-negative integer, got: {:ok, 1}" = error.message
+    end
+  end
+
+  require Record
+  Record.defrecordp(:vec, x: 0, y: 0, z: 0)
+
+  test "assert_receive should not expand argument" do
+    {x, y, z} = {1, 2, 3}
+    send(self(), vec(x: x, y: y, z: z))
+    assert_receive vec(x: ^x, y: ^y)
+  end
+
   test "assert received does not wait" do
-    send self, :hello
+    send(self(), :hello)
     :hello = assert_received :hello
   end
 
   @received :hello
 
   test "assert received with module attribute" do
-    send self, :hello
+    send(self(), :hello)
     :hello = assert_received @received
   end
 
   test "assert received with pinned variable" do
     status = :valid
-    send self(), {:status, :invalid}
+    send(self(), {:status, :invalid})
+
     try do
       "This should never be tested" = assert_received {:status, ^status}
     rescue
       error in [ExUnit.AssertionError] ->
-        "No message matching {:status, ^status} after 0ms.\n" <>
-        "The following variables were pinned:\n" <>
-        "  status = :valid\n" <>
-        "Process mailbox:\n" <>
-        "  {:status, :invalid}" = error.message
+        """
+        No message matching {:status, ^status} after 0ms.
+        The following variables were pinned:
+          status = :valid
+        Process mailbox:
+          {:status, :invalid}\
+        """ = error.message
     end
   end
 
   test "assert received with multiple identical pinned variables" do
     status = :valid
-    send self(), {:status, :invalid, :invalid}
+    send(self(), {:status, :invalid, :invalid})
+
     try do
-      "This should never be tested" = assert_received {
-        :status,
-        ^status,
-        ^status
-      }
+      "This should never be tested" = assert_received {:status, ^status, ^status}
     rescue
       error in [ExUnit.AssertionError] ->
-        "No message matching {:status, ^status, ^status} after 0ms.\n" <>
-        "The following variables were pinned:\n" <>
-        "  status = :valid\n" <>
-        "Process mailbox:\n" <>
-        "  {:status, :invalid, :invalid}" = error.message
+        """
+        No message matching {:status, ^status, ^status} after 0ms.
+        The following variables were pinned:
+          status = :valid
+        Process mailbox:
+          {:status, :invalid, :invalid}\
+        """ = error.message
     end
   end
 
   test "assert received with multiple unique pinned variables" do
     status = :valid
     other_status = :invalid
-    send self(), {:status, :invalid, :invalid}
+    send(self(), {:status, :invalid, :invalid})
+
     try do
-      "This should never be tested" = assert_received {
-        :status,
-        ^status,
-        ^other_status
-      }
+      "This should never be tested" = assert_received {:status, ^status, ^other_status}
     rescue
       error in [ExUnit.AssertionError] ->
-        "No message matching {:status, ^status, ^other_status} after 0ms.\n" <>
-        "The following variables were pinned:\n" <>
-        "  status = :valid\n" <>
-        "  other_status = :invalid\n" <>
-        "Process mailbox:\n" <>
-        "  {:status, :invalid, :invalid}" = error.message
+        """
+        No message matching {:status, ^status, ^other_status} after 0ms.
+        The following variables were pinned:
+          status = :valid
+          other_status = :invalid
+        Process mailbox:
+          {:status, :invalid, :invalid}\
+        """ = error.message
     end
   end
 
@@ -235,39 +351,53 @@ defmodule ExUnit.AssertionsTest do
   end
 
   test "assert received when different message" do
-    send self, {:message, :not_expected, :at_all}
+    send(self(), {:message, :not_expected, :at_all})
+
     try do
       "This should never be tested" = assert_received :hello
     rescue
       error in [ExUnit.AssertionError] ->
-        "No message matching :hello after 0ms.\n" <>
-        "Process mailbox:\n" <>
-        "  {:message, :not_expected, :at_all}" = error.message
+        """
+        No message matching :hello after 0ms.
+        Process mailbox:
+          {:message, :not_expected, :at_all}\
+        """ = error.message
     end
   end
 
   test "assert received when different message having more than 10 on mailbox" do
-    for i <- 1..11, do: send(self, {:message, i})
+    for i <- 1..11, do: send(self(), {:message, i})
+
     try do
       "This should never be tested" = assert_received x when x == :hello
     rescue
       error in [ExUnit.AssertionError] ->
-        "No message matching x when x == :hello after 0ms.\nProcess mailbox:" <>
-        "\n  {:message, 1}\n  {:message, 2}\n  {:message, 3}" <>
-        "\n  {:message, 4}\n  {:message, 5}\n  {:message, 6}" <>
-        "\n  {:message, 7}\n  {:message, 8}\n  {:message, 9}" <>
-        "\n  {:message, 10}\nShowing only 10 of 11 messages." = error.message
+        """
+        No message matching x when x == :hello after 0ms.
+        Process mailbox:
+          {:message, 1}
+          {:message, 2}
+          {:message, 3}
+          {:message, 4}
+          {:message, 5}
+          {:message, 6}
+          {:message, 7}
+          {:message, 8}
+          {:message, 9}
+          {:message, 10}
+        Showing only 10 of 11 messages.\
+        """ = error.message
     end
   end
 
-  test "assert received leaks" do
-    send self, {:hello, :world}
+  test "assert received binds variables" do
+    send(self(), {:hello, :world})
     assert_received {:hello, world}
     :world = world
   end
 
   test "assert received does not leak external variables used in guards" do
-    send self(), {:hello, :world}
+    send(self(), {:hello, :world})
     guard_world = :world
     assert_received {:hello, world} when world == guard_world
     :world = world
@@ -282,7 +412,8 @@ defmodule ExUnit.AssertionsTest do
   end
 
   test "refute received when equal" do
-    send self, :hello
+    send(self(), :hello)
+
     try do
       "This should never be tested" = refute_received :hello
     rescue
@@ -302,7 +433,7 @@ defmodule ExUnit.AssertionsTest do
       error in [ExUnit.AssertionError] ->
         'foo' = error.left
         'bar' = error.right
-        "'foo' in 'bar'" = error.expr |> Macro.to_string
+        "assert('foo' in 'bar')" = Macro.to_string(error.expr)
     end
   end
 
@@ -315,9 +446,9 @@ defmodule ExUnit.AssertionsTest do
       "This should never be tested" = refute 'foo' in ['foo', 'bar']
     rescue
       error in [ExUnit.AssertionError] ->
-        'foo'          = error.left
+        'foo' = error.left
         ['foo', 'bar'] = error.right
-        "'foo' in ['foo', 'bar']" = error.expr |> Macro.to_string
+        "refute('foo' in ['foo', 'bar'])" = Macro.to_string(error.expr)
     end
   end
 
@@ -335,9 +466,9 @@ defmodule ExUnit.AssertionsTest do
       assert {:ok, _} = error(true)
     rescue
       error in [ExUnit.AssertionError] ->
-        "match (=) failed"       = error.message
-        "{:ok, _} = error(true)" = error.expr |> Macro.to_string
-        "{:error, true}"         = error.right |> Macro.to_string
+        "match (=) failed" = error.message
+        "assert({:ok, _} = error(true))" = Macro.to_string(error.expr)
+        "{:error, true}" = Macro.to_string(error.right)
     end
   end
 
@@ -347,8 +478,8 @@ defmodule ExUnit.AssertionsTest do
     rescue
       error in [ExUnit.AssertionError] ->
         "match (=) failed" = error.message
-        "{:ok, _x} = nil"   = error.expr |> Macro.to_string
-        "nil"              = error.right |> Macro.to_string
+        "assert({:ok, _x} = nil)" = Macro.to_string(error.expr)
+        "nil" = Macro.to_string(error.right)
     end
   end
 
@@ -358,7 +489,7 @@ defmodule ExUnit.AssertionsTest do
     rescue
       error in [ExUnit.AssertionError] ->
         "Expected truthy, got nil" = error.message
-        "_x = nil" = error.expr |> Macro.to_string
+        "assert(_x = nil)" = Macro.to_string(error.expr)
     end
   end
 
@@ -367,7 +498,7 @@ defmodule ExUnit.AssertionsTest do
       "This should never be tested" = refute _ = ok(true)
     rescue
       error in [ExUnit.AssertionError] ->
-        "_ = ok(true)" = error.expr |> Macro.to_string
+        "refute(_ = ok(true))" = Macro.to_string(error.expr)
         "Expected false or nil, got {:ok, true}" = error.message
     end
   end
@@ -401,40 +532,33 @@ defmodule ExUnit.AssertionsTest do
   end
 
   test "assert raise with no error" do
-    "This should never be tested" = assert_raise ArgumentError, fn ->
-      nil
-    end
+    "This should never be tested" = assert_raise ArgumentError, fn -> nil end
   rescue
     error in [ExUnit.AssertionError] ->
       "Expected exception ArgumentError but nothing was raised" = error.message
   end
 
   test "assert raise with error" do
-    error = assert_raise ArgumentError, fn ->
-      raise ArgumentError, "test error"
-    end
-
+    error = assert_raise ArgumentError, fn -> raise ArgumentError, "test error" end
     "test error" = error.message
   end
 
   test "assert raise with some other error" do
-    "This should never be tested" = assert_raise ArgumentError, fn ->
-      Not.Defined.function(1, 2, 3)
-    end
+    "This should never be tested" =
+      assert_raise ArgumentError, fn -> Not.Defined.function(1, 2, 3) end
   rescue
     error in [ExUnit.AssertionError] ->
       "Expected exception ArgumentError but got UndefinedFunctionError " <>
-      "(function Not.Defined.function/3 is undefined (module Not.Defined is not available))" = error.message
+        "(function Not.Defined.function/3 is undefined (module Not.Defined is not available))" =
+        error.message
   end
 
   test "assert raise with some other error includes stacktrace from original error" do
-    "This should never be tested" = assert_raise ArgumentError, fn ->
-      Not.Defined.function(1, 2, 3)
-    end
+    "This should never be tested" =
+      assert_raise ArgumentError, fn -> Not.Defined.function(1, 2, 3) end
   rescue
     ExUnit.AssertionError ->
-      stacktrace = System.stacktrace
-      [{Not.Defined, :function, [1, 2, 3], _} | _] = stacktrace
+      [{Not.Defined, :function, [1, 2, 3], _} | _] = __STACKTRACE__
   end
 
   test "assert raise with Erlang error" do
@@ -443,7 +567,8 @@ defmodule ExUnit.AssertionsTest do
     end
   rescue
     error in [ExUnit.AssertionError] ->
-      "Expected exception SyntaxError but got FunctionClauseError (no function clause matching in :lists.flatten/1)" = error.message
+      "Expected exception SyntaxError but got FunctionClauseError (no function clause matching in :lists.flatten/1)" =
+        error.message
   end
 
   test "assert raise comparing messages (for equality)" do
@@ -452,11 +577,13 @@ defmodule ExUnit.AssertionsTest do
     end
   rescue
     error in [ExUnit.AssertionError] ->
-      "Wrong message for RuntimeError" <>
-      "\nExpected:" <>
-      "\n  \"foo\"" <>
-      "\nGot:" <>
-      "\n  \"bar\"" = error.message
+      """
+      Wrong message for RuntimeError
+      expected:
+        "foo"
+      actual:
+        "bar"\
+      """ = error.message
   end
 
   test "assert raise comparing messages (with a regex)" do
@@ -465,11 +592,26 @@ defmodule ExUnit.AssertionsTest do
     end
   rescue
     error in [ExUnit.AssertionError] ->
-      "Wrong message for RuntimeError" <>
-      "\nExpected:" <>
-      "\n  ~r/ba[zk]/" <>
-      "\nGot:" <>
-      "\n  \"bar\"" = error.message
+      """
+      Wrong message for RuntimeError
+      expected:
+        ~r/ba[zk]/
+      actual:
+        "bar"\
+      """ = error.message
+  end
+
+  test "assert raise with an exception with bad message/1 implementation" do
+    assert_raise BrokenError, fn ->
+      raise BrokenError
+    end
+  rescue
+    error in [ExUnit.AssertionError] ->
+      """
+      Got exception ExUnit.AssertionsTest.BrokenError but it failed to produce a message with:
+
+      ** (RuntimeError) error
+      """ <> _ = error.message
   end
 
   test "assert greater than operator" do
@@ -480,9 +622,9 @@ defmodule ExUnit.AssertionsTest do
     "This should never be tested" = assert 1 > 2
   rescue
     error in [ExUnit.AssertionError] ->
-      1       = error.left
-      2       = error.right
-      "1 > 2" = error.expr |> Macro.to_string
+      1 = error.left
+      2 = error.right
+      "assert(1 > 2)" = Macro.to_string(error.expr)
   end
 
   test "assert less or equal than operator" do
@@ -493,7 +635,7 @@ defmodule ExUnit.AssertionsTest do
     "This should never be tested" = assert 2 <= 1
   rescue
     error in [ExUnit.AssertionError] ->
-      "2 <= 1" = error.expr |> Macro.to_string
+      "assert(2 <= 1)" = Macro.to_string(error.expr)
       2 = error.left
       1 = error.right
   end
@@ -510,15 +652,64 @@ defmodule ExUnit.AssertionsTest do
       "assertion" = error.message
   end
 
+  test "assert lack of equality" do
+    try do
+      "This should never be tested" = assert "one" != "one"
+    rescue
+      error in [ExUnit.AssertionError] ->
+        "Assertion with != failed, both sides are exactly equal" = error.message
+        "one" = error.left
+    end
+
+    try do
+      "This should never be tested" = assert 2 != 2.0
+    rescue
+      error in [ExUnit.AssertionError] ->
+        "Assertion with != failed" = error.message
+        2 = error.left
+        2.0 = error.right
+    end
+  end
+
+  test "refute equality" do
+    try do
+      "This should never be tested" = refute "one" == "one"
+    rescue
+      error in [ExUnit.AssertionError] ->
+        "Refute with == failed, both sides are exactly equal" = error.message
+        "one" = error.left
+    end
+
+    try do
+      "This should never be tested" = refute 2 == 2.0
+    rescue
+      error in [ExUnit.AssertionError] ->
+        "Refute with == failed" = error.message
+        2 = error.left
+        2.0 = error.right
+    end
+  end
+
   test "assert in delta" do
     true = assert_in_delta(1.1, 1.2, 0.2)
   end
 
+  test "assert in delta raises when passing a negative delta" do
+    assert_raise ArgumentError, fn ->
+      assert_in_delta(1.1, 1.2, -0.2)
+    end
+  end
+
+  test "assert in delta works with equal values and a delta of zero" do
+    assert_in_delta(10, 10, 0)
+  end
+
   test "assert in delta error" do
     "This should never be tested" = assert_in_delta(10, 12, 1)
   rescue
     error in [ExUnit.AssertionError] ->
-      "Expected the difference between 10 and 12 (2) to be less than 1" = error.message
+      "Expected the difference between 10 and 12 (2) to be less than or equal to 1" =
+        error.message
   end
 
   test "assert in delta with message" do
@@ -568,11 +759,11 @@ defmodule ExUnit.AssertionsTest do
   end
 
   test "catch_throw with throw" do
-    1 = catch_throw(throw 1)
+    1 = catch_throw(throw(1))
   end
 
   test "catch_exit with exit" do
-    1 = catch_exit(exit 1)
+    1 = catch_exit(exit(1))
   end
 
   test "catch_error with error" do
@@ -580,27 +771,28 @@ defmodule ExUnit.AssertionsTest do
   end
 
   test "flunk" do
-    "This should never be tested" = flunk
+    "This should never be tested" = flunk()
   rescue
     error in [ExUnit.AssertionError] ->
       "Flunked!" = error.message
   end
 
   test "flunk with message" do
-    "This should never be tested" = flunk "This should raise an error"
+    "This should never be tested" = flunk("This should raise an error")
   rescue
     error in [ExUnit.AssertionError] ->
       "This should raise an error" = error.message
   end
 
   test "flunk with wrong argument type" do
-    "This should never be tested" = flunk ["flunk takes a binary, not a list"]
+    "This should never be tested" = flunk(["flunk takes a binary, not a list"])
   rescue
     error ->
-      "no function clause matching in ExUnit.Assertions.flunk/1" = FunctionClauseError.message error
+      "no function clause matching in ExUnit.Assertions.flunk/1" =
+        FunctionClauseError.message(error)
   end
 
-  test "AssertionError message should include nice formatting" do
+  test "AssertionError.message/1 is nicely formatted" do
     assert :a = :b
   rescue
     error in [ExUnit.AssertionError] ->
@@ -608,11 +800,12 @@ defmodule ExUnit.AssertionsTest do
 
 
       match (=) failed
-      code: :a = :b
-      rhs:  :b
+      code:  assert :a = :b
+      right: :b
       """ = Exception.message(error)
   end
 
   defp ok(val), do: {:ok, val}
   defp error(val), do: {:error, val}
+  defp not_equal(left, right), do: left != right
 end
