@@ -1,42 +1,56 @@
 defmodule Logger.Backends.Console do
   @moduledoc false
 
-  use GenEvent
+  @behaviour :gen_event
 
-  defstruct [format: nil, metadata: nil, level: nil, colors: nil, device: nil,
-             max_buffer: nil, buffer_size: 0, buffer: [], ref: nil, output: nil]
+  defstruct buffer: [],
+            buffer_size: 0,
+            colors: nil,
+            device: nil,
+            format: nil,
+            level: nil,
+            max_buffer: nil,
+            metadata: nil,
+            output: nil,
+            ref: nil
 
   def init(:console) do
-    if Process.whereis(:user) do
-      init({:user, []})
+    config = Application.get_env(:logger, :console)
+    device = Keyword.get(config, :device, :user)
+
+    if Process.whereis(device) do
+      {:ok, init(config, %__MODULE__{})}
     else
       {:error, :ignore}
     end
   end
 
-  def init({device, opts}) do
-    {:ok, configure(device, opts, %__MODULE__{})}
+  def init({__MODULE__, opts}) when is_list(opts) do
+    config = configure_merge(Application.get_env(:logger, :console), opts)
+    {:ok, init(config, %__MODULE__{})}
   end
 
   def handle_call({:configure, options}, state) do
-    {:ok, :ok, configure(state.device, options, state)}
+    {:ok, :ok, configure(options, state)}
   end
 
-  def handle_event({_level, gl, _event}, state)
-  when node(gl) != node() do
+  def handle_event({_level, gl, _event}, state) when node(gl) != node() do
     {:ok, state}
   end
 
   def handle_event({level, _gl, {Logger, msg, ts, md}}, state) do
-    %{level: log_level, ref: ref, buffer_size: buffer_size,
-      max_buffer: max_buffer} = state
+    %{level: log_level, ref: ref, buffer_size: buffer_size, max_buffer: max_buffer} = state
+
     cond do
       not meet_level?(level, log_level) ->
         {:ok, state}
+
       is_nil(ref) ->
         {:ok, log_event(level, msg, ts, md, state)}
+
       buffer_size < max_buffer ->
         {:ok, buffer_event(level, msg, ts, md, state)}
+
       buffer_size === max_buffer ->
         state = buffer_event(level, msg, ts, md, state)
         {:ok, await_io(state)}
@@ -56,13 +70,21 @@ defmodule Logger.Backends.Console do
   end
 
   def handle_info({:DOWN, ref, _, pid, reason}, %{ref: ref}) do
-    raise "device #{inspect pid} exited: " <> Exception.format_exit(reason)
+    raise "device #{inspect(pid)} exited: " <> Exception.format_exit(reason)
   end
 
   def handle_info(_, state) do
     {:ok, state}
   end
 
+  def code_change(_old_vsn, state, _extra) do
+    {:ok, state}
+  end
+
+  def terminate(_reason, _state) do
+    :ok
+  end
+
   ## Helpers
 
   defp meet_level?(_lvl, nil), do: true
@@ -71,24 +93,34 @@ defmodule Logger.Backends.Console do
     Logger.compare_levels(lvl, min) != :lt
   end
 
-  defp configure(device, options, state) do
-    config =
-      Application.get_env(:logger, :console, [])
-      |> configure_merge(options)
-
-    if device === :user do
-      Application.put_env(:logger, :console, config)
-    end
-
-    format     = Logger.Formatter.compile Keyword.get(config, :format)
-    level      = Keyword.get(config, :level)
-    metadata   = Keyword.get(config, :metadata, [])
-    colors     = configure_colors(config)
-    max_buffer = Keyword.get(config, :max_buffer, 32)
-    %{state | format: format, metadata: Enum.reverse(metadata),
-              level: level, colors: colors, device: device, max_buffer: max_buffer}
+  defp configure(options, state) do
+    config = configure_merge(Application.get_env(:logger, :console), options)
+    Application.put_env(:logger, :console, config)
+    init(config, state)
   end
 
+  defp init(config, state) do
+    level = Keyword.get(config, :level)
+    device = Keyword.get(config, :device, :user)
+    format = Logger.Formatter.compile(Keyword.get(config, :format))
+    colors = configure_colors(config)
+    metadata = Keyword.get(config, :metadata, []) |> configure_metadata()
+    max_buffer = Keyword.get(config, :max_buffer, 32)
+
+    %{
+      state
+      | format: format,
+        metadata: metadata,
+        level: level,
+        colors: colors,
+        device: device,
+        max_buffer: max_buffer
+    }
+  end
+
+  defp configure_metadata(:all), do: :all
+  defp configure_metadata(metadata), do: Enum.reverse(metadata)
+
   defp configure_merge(env, options) do
     Keyword.merge(env, options, fn
       :colors, v1, v2 -> Keyword.merge(v1, v2)
@@ -98,11 +130,14 @@ defmodule Logger.Backends.Console do
 
   defp configure_colors(config) do
     colors = Keyword.get(config, :colors, [])
-    %{debug: Keyword.get(colors, :debug, :cyan),
+
+    %{
+      debug: Keyword.get(colors, :debug, :cyan),
       info: Keyword.get(colors, :info, :normal),
       warn: Keyword.get(colors, :warn, :yellow),
       error: Keyword.get(colors, :error, :red),
-      enabled: Keyword.get(colors, :enabled, IO.ANSI.enabled?)}
+      enabled: Keyword.get(colors, :enabled, IO.ANSI.enabled?())
+    }
   end
 
   defp log_event(level, msg, ts, md, %{device: device} = state) do
@@ -116,12 +151,13 @@ defmodule Logger.Backends.Console do
     %{state | buffer: buffer, buffer_size: buffer_size + 1}
   end
 
-  defp async_io(:user, output) do
-    case Process.whereis(:user) do
+  defp async_io(name, output) when is_atom(name) do
+    case Process.whereis(name) do
       device when is_pid(device) ->
         async_io(device, output)
+
       nil ->
-        raise "no device registered with the name :user"
+        raise "no device registered with the name #{inspect(name)}"
     end
   end
 
@@ -137,43 +173,49 @@ defmodule Logger.Backends.Console do
     receive do
       {:io_reply, ^ref, :ok} ->
         handle_io_reply(:ok, state)
+
       {:io_reply, ^ref, error} ->
         handle_io_reply(error, state)
         |> await_io()
+
       {:DOWN, ^ref, _, pid, reason} ->
-        raise "device #{inspect pid} exited: " <> Exception.format_exit(reason)
+        raise "device #{inspect(pid)} exited: " <> Exception.format_exit(reason)
     end
   end
 
   defp format_event(level, msg, ts, md, state) do
     %{format: format, metadata: keys, colors: colors} = state
+
     format
     |> Logger.Formatter.format(level, msg, ts, take_metadata(md, keys))
     |> color_event(level, colors, md)
   end
 
+  defp take_metadata(metadata, :all) do
+    Keyword.drop(metadata, [:crash_reason])
+  end
+
   defp take_metadata(metadata, keys) do
-    Enum.reduce keys, [], fn key, acc ->
+    Enum.reduce(keys, [], fn key, acc ->
       case Keyword.fetch(metadata, key) do
         {:ok, val} -> [{key, val} | acc]
-        :error     -> acc
+        :error -> acc
       end
-    end
+    end)
   end
 
   defp color_event(data, _level, %{enabled: false}, _md), do: data
 
   defp color_event(data, level, %{enabled: true} = colors, md) do
     color = md[:ansi_color] || Map.fetch!(colors, level)
-    [IO.ANSI.format_fragment(color, true), data | IO.ANSI.reset]
+    [IO.ANSI.format_fragment(color, true), data | IO.ANSI.reset()]
   end
 
   defp log_buffer(%{buffer_size: 0, buffer: []} = state), do: state
 
   defp log_buffer(state) do
     %{device: device, buffer: buffer} = state
-    %{state | ref: async_io(device, buffer), buffer: [], buffer_size: 0,
-      output: buffer}
+    %{state | ref: async_io(device, buffer), buffer: [], buffer_size: 0, output: buffer}
   end
 
   defp handle_io_reply(:ok, %{ref: ref} = state) do
@@ -195,10 +237,18 @@ defmodule Logger.Backends.Console do
 
   defp retry_log(error, %{device: device, ref: ref, output: dirty} = state) do
     Process.demonitor(ref, [:flush])
-    case :unicode.characters_to_binary(dirty) do
+
+    try do
+      :unicode.characters_to_binary(dirty)
+    rescue
+      ArgumentError ->
+        clean = ["failure while trying to log malformed data: ", inspect(dirty), ?\n]
+        %{state | ref: async_io(device, clean), output: clean}
+    else
       {_, good, bad} ->
         clean = [good | Logger.Formatter.prune(bad)]
         %{state | ref: async_io(device, clean), output: clean}
+
       _ ->
         # A well behaved IO device should not error on good data
         raise "failure while logging consoles messages: " <> inspect(error)
