@@ -14,11 +14,13 @@ defmodule Mix.SCM.Git do
     case opts[:lock] do
       {:git, _, lock_rev, lock_opts} ->
         lock = String.slice(lock_rev, 0, 7)
-        case Enum.find_value [:branch, :ref, :tag], &List.keyfind(lock_opts, &1, 0) do
-          {:ref, _}  -> lock <> " (ref)"
+
+        case Enum.find_value([:branch, :ref, :tag], &List.keyfind(lock_opts, &1, 0)) do
+          {:ref, _} -> lock <> " (ref)"
           {key, val} -> lock <> " (#{key}: #{val})"
-          nil        -> lock
+          nil -> lock
         end
+
       _ ->
         nil
     end
@@ -36,9 +38,11 @@ defmodule Mix.SCM.Git do
         |> Keyword.delete(:github)
         |> Keyword.put(:git, "https://github.com/#{gh}.git")
         |> validate_git_options
+
       opts[:git] ->
         opts
         |> validate_git_options
+
       true ->
         nil
     end
@@ -48,7 +52,7 @@ defmodule Mix.SCM.Git do
     # Are we inside a Git repository?
     opts[:checkout]
     |> Path.join(".git/HEAD")
-    |> File.regular?
+    |> File.regular?()
   end
 
   def lock_status(opts) do
@@ -59,22 +63,24 @@ defmodule Mix.SCM.Git do
       lock_rev = get_lock_rev(lock, opts) ->
         File.cd!(opts[:checkout], fn ->
           %{origin: origin, rev: rev} = get_rev_info()
+
           if get_lock_repo(lock) == origin and lock_rev == rev do
             :ok
           else
             :mismatch
           end
         end)
+
       is_nil(lock) ->
         :mismatch
+
       true ->
         :outdated
     end
   end
 
   def equal?(opts1, opts2) do
-    opts1[:git] == opts2[:git] and
-      get_lock_opts(opts1) == get_lock_opts(opts2)
+    opts1[:git] == opts2[:git] and get_lock_opts(opts1) == get_lock_opts(opts2)
   end
 
   def managers(_opts) do
@@ -86,9 +92,10 @@ defmodule Mix.SCM.Git do
     path = opts[:checkout]
     File.rm_rf!(path)
     File.mkdir_p!(path)
+
     File.cd!(path, fn ->
-      git!("init --quiet")
-      git!("--git-dir=.git remote add origin \"#{opts[:git]}\"")
+      git!(["init", "--quiet"])
+      git!(["--git-dir=.git", "remote", "add", "origin", opts[:git]])
       checkout(path, opts)
     end)
   end
@@ -96,26 +103,28 @@ defmodule Mix.SCM.Git do
   def update(opts) do
     assert_git!()
     path = opts[:checkout]
-    File.cd! path, fn -> checkout(path, opts) end
+    File.cd!(path, fn -> checkout(path, opts) end)
   end
 
   defp checkout(_path, opts) do
+    Mix.shell().print_app()
+
     # Set configuration
     sparse_toggle(opts)
     update_origin(opts[:git])
 
     # Fetch external data
-    command = IO.iodata_to_binary(["--git-dir=.git fetch --force --quiet",
-                                   progress_switch(git_version()),
-                                   tags_switch(opts[:tag])])
-    git!(command)
+    ["--git-dir=.git", "fetch", "--force", "--quiet"]
+    |> Kernel.++(progress_switch(git_version()))
+    |> Kernel.++(tags_switch(opts[:tag]))
+    |> git!()
 
-    # Migrate the git repo
+    # Migrate the Git repo
     rev = get_lock_rev(opts[:lock], opts) || get_opts_rev(opts)
-    git!("--git-dir=.git checkout --quiet #{rev}")
+    git!(["--git-dir=.git", "checkout", "--quiet", rev])
 
     if opts[:submodules] do
-      git!("--git-dir=.git submodule update --init --recursive")
+      git!(["--git-dir=.git", "submodule", "update", "--init", "--recursive"])
     end
 
     # Get the new repo lock
@@ -135,45 +144,54 @@ defmodule Mix.SCM.Git do
     cond do
       sparse = opts[:sparse] ->
         sparse_check(git_version())
-        git!("--git-dir=.git config core.sparsecheckout true")
+        git!(["--git-dir=.git", "config", "core.sparsecheckout", "true"])
+        File.mkdir_p!(".git/info")
         File.write!(".git/info/sparse-checkout", sparse)
+
       File.exists?(".git/info/sparse-checkout") ->
         File.write!(".git/info/sparse-checkout", "*")
-        git!("--git-dir=.git read-tree -mu HEAD")
-        git!("--git-dir=.git config core.sparsecheckout false")
+        git!(["--git-dir=.git", "read-tree", "-mu", "HEAD"])
+        git!(["--git-dir=.git", "config", "core.sparsecheckout", "false"])
         File.rm(".git/info/sparse-checkout")
+
       true ->
         :ok
     end
   end
 
   defp sparse_check(version) do
-    unless {1, 7, 0} <= version do
-      version = version |> Tuple.to_list |> Enum.join(".")
-      Mix.raise "Git >= 1.7.0 is required to use sparse checkout. " <>
-                "You are running version #{version}"
+    unless {1, 7, 4} <= version do
+      version = version |> Tuple.to_list() |> Enum.join(".")
+
+      Mix.raise(
+        "Git >= 1.7.4 is required to use sparse checkout. " <>
+          "You are running version #{version}"
+      )
     end
   end
 
-  defp progress_switch(version) when {1, 7, 1} <= version, do: " --progress"
-  defp progress_switch(_),                                 do: ""
+  defp progress_switch(version) do
+    if {1, 7, 1} <= version, do: ["--progress"], else: []
+  end
 
-  defp tags_switch(nil), do: ""
-  defp tags_switch(_), do: " --tags"
+  defp tags_switch(nil), do: []
+  defp tags_switch(_), do: ["--tags"]
 
   ## Helpers
 
   defp validate_git_options(opts) do
-    err = "You should specify only one of branch, ref or tag, and only once. " <>
-          "Error on Git dependency: #{opts[:git]}"
+    err =
+      "You should specify only one of branch, ref or tag, and only once. " <>
+        "Error on Git dependency: #{opts[:git]}"
+
     validate_single_uniq(opts, [:branch, :ref, :tag], err)
   end
 
   defp validate_single_uniq(opts, take, error) do
     case Keyword.take(opts, take) do
-      []  -> opts
+      [] -> opts
       [_] -> opts
-      _   -> Mix.raise error
+      _ -> Mix.raise(error)
     end
   end
 
@@ -189,6 +207,7 @@ defmodule Mix.SCM.Git do
       lock
     end
   end
+
   defp get_lock_rev(_, _), do: nil
 
   defp get_lock_opts(opts) do
@@ -210,49 +229,65 @@ defmodule Mix.SCM.Git do
   end
 
   defp get_rev_info do
-    destructure [origin, rev],
-      :os.cmd('git --git-dir=.git config remote.origin.url && git --git-dir=.git rev-parse --verify --quiet HEAD')
-      |> IO.iodata_to_binary
-      |> String.split("\n", trim: true)
-    %{origin: origin, rev: rev}
+    # These commands can fail and we don't want to raise.
+    origin_command = ["--git-dir=.git", "config", "remote.origin.url"]
+    rev_command = ["--git-dir=.git", "rev-parse", "--verify", "--quiet", "HEAD"]
+
+    with {origin, 0} <- System.cmd("git", origin_command),
+         {rev, 0} <- System.cmd("git", rev_command) do
+      %{origin: String.trim(origin), rev: String.trim(rev)}
+    else
+      _ -> %{origin: nil, rev: nil}
+    end
   end
 
   defp update_origin(location) do
-    git!(~s(--git-dir=.git config remote.origin.url "#{location}"))
+    git!(["--git-dir=.git", "config", "remote.origin.url", location])
     :ok
   end
 
-  defp git!(command) do
-    if Mix.shell.cmd("git " <> command) != 0 do
-      Mix.raise "Command \"git #{command}\" failed"
+  defp git!(args, into \\ default_into()) do
+    case System.cmd("git", args, into: into, stderr_to_stdout: true) do
+      {response, 0} -> response
+      {_, _} -> Mix.raise("Command \"git #{Enum.join(args, " ")}\" failed")
+    end
+  end
+
+  defp default_into() do
+    case Mix.shell() do
+      Mix.Shell.IO -> IO.stream(:stdio, :line)
+      _ -> ""
     end
-    :ok
   end
 
   defp assert_git! do
     case Mix.State.fetch(:git_available) do
       {:ok, true} ->
         :ok
+
       :error ->
         if System.find_executable("git") do
           Mix.State.put(:git_available, true)
         else
-          Mix.raise "Error fetching/updating Git repository: the \"git\" "  <>
-            "executable is not available in your PATH. Please install "   <>
-            "Git on this machine or pass --no-deps-check if you want to " <>
-            "run a previously built application on a system without Git."
+          Mix.raise(
+            "Error fetching/updating Git repository: the \"git\" " <>
+              "executable is not available in your PATH. Please install " <>
+              "Git on this machine or pass --no-deps-check if you want to " <>
+              "run a previously built application on a system without Git."
+          )
         end
     end
   end
 
-  defp git_version do
+  def git_version do
     case Mix.State.fetch(:git_version) do
       {:ok, version} ->
         version
+
       :error ->
         version =
-          :os.cmd('git --version')
-          |> IO.iodata_to_binary
+          ["--version"]
+          |> git!("")
           |> parse_version
 
         Mix.State.put(:git_version, version)
@@ -261,10 +296,11 @@ defmodule Mix.SCM.Git do
   end
 
   defp parse_version("git version " <> version) do
-    String.split(version, ".")
+    version
+    |> String.split(".")
     |> Enum.take(3)
     |> Enum.map(&to_integer/1)
-    |> List.to_tuple
+    |> List.to_tuple()
   end
 
   defp to_integer(string) do
