@@ -2,14 +2,18 @@ defmodule Mix.State do
   @moduledoc false
   @name __MODULE__
 
-  def start_link() do
-    Agent.start_link(__MODULE__, :init, [], [name: @name])
+  use Agent
+
+  def start_link(_opts) do
+    Agent.start_link(__MODULE__, :init, [], name: @name)
   end
 
   def init() do
-    %{shell: Mix.Shell.IO,
+    %{
+      shell: Mix.Shell.IO,
       env: String.to_atom(System.get_env("MIX_ENV") || "dev"),
-      scm: [Mix.SCM.Git, Mix.SCM.Path]}
+      scm: [Mix.SCM.Git, Mix.SCM.Path]
+    }
   end
 
   def fetch(key) do
