@@ -17,38 +17,40 @@ defmodule Mix.Tasks.Clean do
 
   @switches [deps: :boolean, only: :string]
 
-  @spec run(OptionParser.argv) :: :ok
   def run(args) do
-    Mix.Project.get!
-    loadpaths!
+    Mix.Project.get!()
+    loadpaths!()
 
     {opts, _, _} = OptionParser.parse(args, switches: @switches)
 
-    _ = for compiler <- [:protocols] ++ Mix.Tasks.Compile.compilers(),
-            module = Mix.Task.get("compile.#{compiler}"),
-            function_exported?(module, :clean, 0),
-            do: module.clean
+    _ =
+      for compiler <- [:protocols] ++ Mix.Tasks.Compile.compilers(),
+          module = Mix.Task.get("compile.#{compiler}"),
+          function_exported?(module, :clean, 0),
+          do: module.clean
 
-    build = Mix.Project.build_path
-            |> Path.dirname
-            |> Path.join("#{opts[:only] || :*}")
+    build =
+      Mix.Project.build_path()
+      |> Path.dirname()
+      |> Path.join("#{opts[:only] || :*}")
 
     if opts[:deps] do
       build
-      |> Path.wildcard
+      |> Path.wildcard()
       |> Enum.each(&File.rm_rf/1)
     else
       build
-      |> Path.join("lib/#{Mix.Project.config[:app]}")
-      |> Path.wildcard
+      |> Path.join("lib/#{Mix.Project.config()[:app]}")
+      |> Path.wildcard()
       |> Enum.each(&File.rm_rf/1)
     end
   end
 
   # Loadpaths without checks because compilers may be defined in deps.
   defp loadpaths! do
-    Mix.Task.run "loadpaths", ["--no-elixir-version-check", "--no-deps-check", "--no-archives-check"]
-    Mix.Task.reenable "loadpaths"
-    Mix.Task.reenable "deps.loadpaths"
+    flags = ["--no-elixir-version-check", "--no-deps-check", "--no-archives-check"]
+    Mix.Task.run("loadpaths", flags)
+    Mix.Task.reenable("loadpaths")
+    Mix.Task.reenable("deps.loadpaths")
   end
 end
