@@ -1,7 +1,7 @@
 defmodule Mix.Tasks.Local.PublicKeys do
   use Mix.Task
 
-  @shortdoc  "Manages public keys"
+  @shortdoc "Manages public keys"
 
   @moduledoc """
   Public keys are used by Mix to install packages like Rebar and Hex.
@@ -31,50 +31,55 @@ defmodule Mix.Tasks.Local.PublicKeys do
 
     * `--force` - forces installation without a shell prompt; primarily
       intended for automation in build systems like `make`
+
   """
-  @spec run(OptionParser.argv) :: true
+
   def run(argv) do
-    {opts, argv} =
-      OptionParser.parse!(argv, switches: [force: :boolean, detailed: :boolean])
+    {opts, argv} = OptionParser.parse!(argv, switches: [force: :boolean, detailed: :boolean])
 
     case argv do
-      []         -> show(opts)
+      [] -> show(opts)
       [path | _] -> install(path, opts)
     end
   end
 
   defp show(opts) do
-    for {id, key} <- Mix.PublicKey.public_keys do
-      Mix.shell.info "* #{id}"
+    for {id, key} <- Mix.PublicKey.public_keys() do
+      Mix.shell().info("* #{id}")
+
       if opts[:detailed] do
-        Mix.shell.info "\n#{key}"
+        Mix.shell().info("\n#{key}")
       end
     end
 
-    Mix.shell.info "Public keys (except in-memory ones) installed at: #{Mix.PublicKey.public_keys_path()}"
+    Mix.shell().info(
+      "Public keys (except in-memory ones) installed at: #{Mix.PublicKey.public_keys_path()}"
+    )
   end
 
   defp install(source, opts) do
-    key  = File.read!(source)
+    key = File.read!(source)
     base = Path.basename(source)
-    dest = Path.join(Mix.PublicKey.public_keys_path, base)
+    dest = Path.join(Mix.PublicKey.public_keys_path(), base)
 
     # Validate the key is good
     _ = Mix.PublicKey.decode!(source, key)
 
     if opts[:force] || should_install?(source, dest) do
-      File.mkdir_p!(Mix.PublicKey.public_keys_path)
+      File.mkdir_p!(Mix.PublicKey.public_keys_path())
       File.write!(dest, key)
-      Mix.shell.info [:green, "* creating ", :reset, Path.relative_to_cwd(dest)]
+      Mix.shell().info([:green, "* creating ", :reset, Path.relative_to_cwd(dest)])
     end
   end
 
   defp should_install?(source, dest) do
     if File.exists?(dest) do
-      Mix.shell.yes?("There is already a public key named #{Path.basename(dest)}.\n" <>
-                     "Are you sure you want to replace it?")
+      Mix.shell().yes?(
+        "There is already a public key named #{Path.basename(dest)}.\n" <>
+          "Are you sure you want to replace it?"
+      )
     else
-      Mix.shell.yes?("Are you sure you want to install public key #{source}?")
+      Mix.shell().yes?("Are you sure you want to install public key #{source}?")
     end
   end
 end
