@@ -14,30 +14,33 @@ defmodule Mix.Tasks.Profile.Fprof do
   with all processes which are spawned by it. Other processes (e.g. those
   residing in the OTP application supervision tree) are not profiled.
 
-  To profile the code, you can use the syntax similar to `mix run` task:
+  To profile the code, you can use syntax similar to the `mix run` task:
 
       mix profile.fprof -e Hello.world
       mix profile.fprof my_script.exs arg1 arg2 arg3
 
+  This task is automatically reenabled, so you can profile multiple times
+  in the same Mix invocation.
+
   ## Command line options
 
-    * `--callers`       - print detailed information about immediate callers and called functions
-    * `--details`       - include profile data for each profiled process
-    * `--sort key`      - sort the output by given key: acc (default) or own
+    * `--callers` - prints detailed information about immediate callers and called functions
+    * `--details` - includes profile data for each profiled process
+    * `--sort key` - sorts the output by given key: `acc` (default) or `own`
     * `--config`, `-c`  - loads the given configuration file
-    * `--eval`, `-e`    - evaluate the given code
-    * `--require`, `-r` - require pattern before running the command
-    * `--parallel-require`, `-pr` - requires pattern in parallel
-    * `--no-compile`    - do not compile even if files require compilation
-    * `--no-deps-check` - do not check dependencies
-    * `--no-archives-check` - do not check archives
-    * `--no-start`      - do not start applications after compilation
-    * `--no-elixir-version-check` - do not check the Elixir version from mix.exs
-    * `--no-warmup`     - do not execute code once before profiling
+    * `--eval`, `-e` - evaluates the given code
+    * `--require`, `-r` - requires pattern before running the command
+    * `--parallel`, `-p` - makes all requires parallel
+    * `--no-compile`    - does not compile even if files require compilation
+    * `--no-deps-check` - does not check dependencies
+    * `--no-archives-check` - does not check archives
+    * `--no-start` - does not start applications after compilation
+    * `--no-elixir-version-check` - does not check the Elixir version from mix.exs
+    * `--no-warmup` - does not execute code once before profiling
 
   ## Profile output
 
-  The example output looks as following:
+  Example output:
       #                                        CNT    ACC (ms)    OWN (ms)
       Total                                 200279    1972.188    1964.579
       :fprof.apply_start_stop/4                  0    1972.188       0.012
@@ -50,136 +53,92 @@ defmodule Mix.Tasks.Profile.Fprof do
   The default output contains data gathered from all profiled processes.
   All times are wall clock milliseconds. The columns have the following meaning:
 
-    * CNT - total number of invocation of the given function
+    * CNT - total number of invocations of the given function
     * ACC - total time spent in the function
     * OWN - time spent in the function, excluding the time of called functions
 
   The first row (Total) is the sum of all functions executed in all profiled
-  processes. For the given output, we had in total 200279 function calls and spent
-  about 2 seconds running the entire code.
+  processes. For the given output, we had a total of 200279 function calls and spent
+  about 2 seconds running the code.
 
-  You can obtain further information if you provide `--callers` and
+  More detailed information is returned if you provide the `--callers` and
   `--details` options.
 
   When `--callers` option is specified, you'll see expanded function entries:
 
-      Mod.caller_1/0                            3     200.000       0.017
-      Mod.caller_2/0                            2     100.000       0.017
+      Mod.caller1/0                             3     200.000       0.017
+      Mod.caller2/0                             2     100.000       0.017
         Mod.some_function/0                     5     300.000       0.017  <--
-          Mod.called_1/0                        4     250.000       0.010
-          Mod.called_2/0                        1      50.000       0.030
+          Mod.called1/0                         4     250.000       0.010
+          Mod.called2/0                         1      50.000       0.030
 
   Here, the arrow (`<--`) indicates the __marked__ function - the function
   described by this paragraph. You also see its immediate callers (above) and
   called functions (below).
 
   All the values of caller functions describe the marked function. For example,
-  the first row means that `Mod.caller_1/0` invoked `Mod.some_function/0` 3 times.
-  200ms of the total time spent in `Mod.some_function/0` happened due to this
-  particular caller.
+  the first row means that `Mod.caller1/0` invoked `Mod.some_function/0` 3 times.
+  200ms of the total time spent in `Mod.some_function/0` was spent processing
+  calls from this particular caller.
 
   In contrast, the values for the called functions describe those functions, but
   in the context of the marked function. For example, the last row means that
-  `Mod.called_2/0` was called once by `Mod.some_function/0`, and in that case
+  `Mod.called2/0` was called once by `Mod.some_function/0`, and in that case
   the total time spent in the function was 50ms.
 
   For a detailed explanation it's worth reading the analysis in
-  [Erlang documentation for fprof](http://www.erlang.org/doc/man/fprof.html#analysis).
+  [Erlang/OTP documentation for fprof](http://www.erlang.org/doc/man/fprof.html#analysis).
 
   ## Caveats
 
   You should be aware that the code being profiled is running in an anonymous
-  function which is invoked by `:fprof` module. Thus, you'll see some additional
-  entries in your profile output, such as `:fprof` calls, an anonymous
+  function which is invoked by [`:fprof` module](http://wwww.erlang.org/doc/man/fprof.html).
+  Thus, you'll see some additional entries in your profile output,
+  such as `:fprof` calls, an anonymous
   function with high ACC time, or an `:undefined` function which represents
   the outer caller (non-profiled code which started the profiler).
 
   Also, keep in mind that profiling might significantly increase the running time
-  of the profiled processes. This might skew your results for example if those
+  of the profiled processes. This might skew your results if, for example, those
   processes perform some I/O operations, since running time of those operations
   will remain unchanged, while CPU bound operations of the profiled processes
   might take significantly longer. Thus, when profiling some intensive program,
   try to reduce such dependencies, or be aware of the resulting bias.
 
   Finally, it's advised to profile your program with the `prod` environment, since
-  this should give you a more correct insight into your real bottlenecks.
+  this should provide more realistic insights into bottlenecks.
   """
 
-  @switches [parallel_require: :keep, require: :keep, eval: :keep, config: :keep,
-             compile: :boolean, deps_check: :boolean, start: :boolean, archives_check: :boolean,
-             details: :boolean, callers: :boolean, sort: :string, elixir_version_check: :boolean,
-             warmup: :boolean]
+  @switches [
+    parallel: :boolean,
+    require: :keep,
+    eval: :keep,
+    config: :keep,
+    compile: :boolean,
+    deps_check: :boolean,
+    start: :boolean,
+    archives_check: :boolean,
+    details: :boolean,
+    callers: :boolean,
+    sort: :string,
+    elixir_version_check: :boolean,
+    warmup: :boolean,
+    parallel_require: :keep
+  ]
+
+  @aliases [r: :require, p: :parallel, e: :eval, c: :config]
 
-  @spec run(OptionParser.argv) :: :ok
   def run(args) do
-    {opts, head} = OptionParser.parse_head!(args,
-      aliases: [r: :require, pr: :parallel_require, e: :eval, c: :config],
-      strict: @switches)
+    {opts, head} = OptionParser.parse_head!(args, aliases: @aliases, strict: @switches)
+    Mix.Task.reenable("profile.fprof")
 
-    {file, argv} =
-      case {Keyword.has_key?(opts, :eval), head} do
-        {true, _}    -> {nil, head}
-        {_, [h | t]} -> {h, t}
-        {_, []}      -> {nil, []}
-      end
-
-    System.argv(argv)
-    process_config opts
-
-    # Start app after rewriting System.argv,
-    # but before requiring and evaling
-    unless "--no-start" in args do
-      Mix.Task.run "app.start", args
-    end
-    process_load opts
-
-    _ = if file do
-      if File.regular?(file) do
-        profile_code(File.read!(file), opts)
-      else
-        Mix.raise "No such file: #{file}"
-      end
-    end
-
-    :ok
-  end
-
-  defp process_config(opts) do
-    Enum.each opts, fn
-      {:config, value} ->
-        Mix.Task.run "loadconfig", [value]
-      _ ->
-        :ok
-    end
-  end
-
-  defp process_load(opts) do
-    Enum.each opts, fn
-      {:parallel_require, value} ->
-        case filter_patterns(value) do
-          [] ->
-            Mix.raise "No files matched pattern #{inspect value} given to --parallel-require"
-          filtered ->
-            Kernel.ParallelRequire.files(filtered)
-        end
-      {:require, value} ->
-        case filter_patterns(value) do
-          [] ->
-            Mix.raise "No files matched pattern #{inspect value} given to --require"
-          filtered ->
-            Enum.each(filtered, &Code.require_file(&1))
-        end
-      {:eval, value} ->
-        profile_code(value, opts)
-      _ ->
-        :ok
-    end
-  end
-
-  defp filter_patterns(pattern) do
-    Path.wildcard(pattern)
-    |> Enum.uniq
-    |> Enum.filter(&File.regular?/1)
+    Mix.Tasks.Run.run(
+      ["--no-mix-exs" | args],
+      opts,
+      head,
+      &profile_code(&1, opts),
+      &profile_code(File.read!(&1), opts)
+    )
   end
 
   # Profiling functions
@@ -187,16 +146,34 @@ defmodule Mix.Tasks.Profile.Fprof do
   defp profile_code(code_string, opts) do
     content =
       quote do
-        unquote(__MODULE__).profile(fn ->
-          unquote(Code.string_to_quoted!(code_string))
-        end, unquote(opts))
+        unquote(__MODULE__).profile(
+          fn ->
+            unquote(Code.string_to_quoted!(code_string))
+          end,
+          unquote(Macro.escape(Enum.map(opts, &parse_opt/1)))
+        )
       end
+
     # Use compile_quoted since it leaves less noise than eval_quoted
     Code.compile_quoted(content)
   end
 
-  @doc false
-  def profile(fun, opts) do
+  defp parse_opt({:sort, "acc"}), do: {:sort, :acc}
+  defp parse_opt({:sort, "own"}), do: {:sort, :own}
+  defp parse_opt({:sort, other}), do: Mix.raise("Invalid sort option: #{other}")
+  defp parse_opt(other), do: other
+
+  @doc """
+  Allows to programatically run the `fprof` profiler on expression in `fun`.
+
+  ## Options
+
+    * `:callers` - prints detailed information about immediate callers and called functions
+    * `:details` - includes profile data for each profiled process
+    * `:sort` - sorts the output by given key: `:acc` (default) or `:own`
+
+  """
+  def profile(fun, opts \\ []) when is_function(fun, 0) do
     fun
     |> profile_and_analyse(opts)
     |> print_output
@@ -204,26 +181,22 @@ defmodule Mix.Tasks.Profile.Fprof do
 
   defp profile_and_analyse(fun, opts) do
     if Keyword.get(opts, :warmup, true) do
-      IO.puts "Warmup..."
+      IO.puts("Warmup...")
       fun.()
     end
 
-    sorting = case Keyword.get(opts, :sort, "acc") do
-      "acc" -> :acc
-      "own" -> :own
-    end
-
     {:ok, tracer} = :fprof.profile(:start)
     :fprof.apply(fun, [], tracer: tracer)
 
     {:ok, analyse_dest} = StringIO.open("")
+
     try do
       :fprof.analyse(
         dest: analyse_dest,
         totals: true,
         details: Keyword.get(opts, :details, false),
         callers: Keyword.get(opts, :callers, false),
-        sort: sorting
+        sort: Keyword.get(opts, :sort, :acc)
       )
     else
       :ok ->
@@ -251,19 +224,22 @@ defmodule Mix.Tasks.Profile.Fprof do
             {:ok, term} = :erl_parse.parse_term(tokens)
             {term, leftover}
 
-          {:eof, _} -> nil
+          {:eof, _} ->
+            nil
         end
-      _ -> nil
+
+      _ ->
+        nil
     end
   end
 
   defp print_total_row([{:totals, count, acc, own}]) do
-    IO.puts ""
+    IO.puts("")
     print_row(["s", "s", "s", "s", "s"], ["", "CNT", "ACC (ms)", "OWN (ms)", ""])
     print_row(["s", "B", ".3f", ".3f", "s"], ["Total", count, acc, own, ""])
   end
 
-  # Represents the "pid" entry
+  # Represents the "PID" entry
   defp print_analysis_result([{pid_atom, count, :undefined, own} | info]) do
     print_process(pid_atom, count, own)
 
@@ -303,10 +279,8 @@ defmodule Mix.Tasks.Profile.Fprof do
   end
 
   defp print_function({fun, count, acc, own}, prefix \\ "", suffix \\ "") do
-    print_row(
-      ["s", "B", ".3f", ".3f", "s"],
-      ["#{prefix}#{function_text(fun)}", count, acc, own, suffix]
-    )
+    text = "#{prefix}#{function_text(fun)}"
+    print_row(["s", "B", ".3f", ".3f", "s"], [text, count, acc, own, suffix])
   end
 
   defp function_text({module, function, arity}) do
@@ -318,10 +292,10 @@ defmodule Mix.Tasks.Profile.Fprof do
   @columns [-60, 10, 12, 12, 5]
   defp print_row(formats, data) do
     Stream.zip(@columns, formats)
-    |> Stream.map(fn({width, format}) -> "~#{width}#{format}" end)
-    |> Enum.join
+    |> Stream.map(fn {width, format} -> "~#{width}#{format}" end)
+    |> Enum.join()
     |> :io.format(data)
 
-    IO.puts ""
+    IO.puts("")
   end
 end
